@echo off
setlocal enabledelayedexpansion
REM (c)2013 Schwindt CAD/CAM-Technologie GmbH
REM
REM  Name              : cabum.bat
REM  Version           : 2.11
REM
REM  Created           : 26.04.2005 (Walter Laub)
REM  Last Modification : 28.03.2018 mrd
REM
REM  Intention         : starts CABUM on Windows
REM                      all configuration is normally replicated from a server
REM
REM  HISTORY :
REM
REM		1.33,	wla: Link-copy and Perl-Installation changed
REM				(automatic check for administrator rights)
REM		1.33a,	wla: More documentation
REM		1.33b,	wla: again the Perl-Installation ...
REM		1.40,	wla: first VISTA changes, Perl64
REM		1.41,	mrb: bergabeparameter %CABUM_CONFIG_LOC% hinzugefgt
REM		1.42	mrb: bergabeparameter %APPDATA% hinzugefgt
REM		1.43	mrd: Parameter -replicate und -forcereplicate hinzugefgt
REM		1.44	mrd: WakeOnLAN
REM		1.50	mrd: Multithreaded copy und Replikation auf Basis installierter Catia's
REM		1.51	mrd: Multithreaded copy Directory Listing added
REM		1.52	mrd: Robocopy-detection changed
REM		1.53	wla: Support for 2 customer scripts (batch-files) before starting GUI
REM		2.00	mrd: Complete rewrite. Settings from configuration-file. More complex
REM                  functions rewritten in Perl.
REM		2.01	wla: CABUM_HOME and CABUM_CONFIG with %~dp0 instead %cd%
REM     2.02    wla: Bugfixes
REM     2.03    wla: Support for slave_in_update.txt (MASTER to SLAVE replication)
REM     2.04    mrd: Modified link creation
REM		2.05	mrd: Path to find.exe added
REM		2.06	mrd: Modified Java detection
REM		2.08	mrd: Added Windows 10 Support
REM		2.09	mrd: c:\3dx will be created and compressed if not exists
REM     2.10    mrd: More robust about spaces in pathnames
REM		2.11	mrd: Switch to CABUM 7.10v and Java 9; no 32-bit support
REM     2.12	lnt: New reading method of cabum_config.txt
REM 	2.13	lnt: Modified PATH Variable and added file extensions (.exe)

rem set find=%SystemRoot%\System32\find.exe
set PATH=%SystemRoot%\System32;%PATH%

REM Check and execute earliest customer batch
if exist "%~dp0..\..\config\cabum-pre-earliest-customer.bat" set custscript1=%~dp0..\..\config\cabum-pre-earliest-customer.bat
if exist "%~dp0..\..\..\..\..\catia\sct\fdec\site\all\cabum\cabum-pre-earliest-customer.bat" set custscript1=%~dp0..\..\..\..\..\catia\sct\fdec\site\all\cabum\cabum-pre-earliest-customer.bat
if exist "%~dp0..\..\..\..\my\cabum\cabum-pre-earliest-customer.bat" set custscript1=%~dp0..\..\..\..\my\cabum\cabum-pre-earliest-customer.bat
if exist "!custscript1!" (
	echo.
	echo Execute customer first batch file "!custscript1!"
	echo.
	call "!custscript1!"
)

REM Workaround if %temp% does not exist
if "!temp!"=="" (
	if not exist "!systemdrive!\temp" mkdir "!systemdrive!\temp"
	set temp=!systemdrive!\temp
)

REM Reads and sets the config
echo.|set /p =Searching config...
if exist "%~dp0..\config\cabum_config.txt" set CABUM_BAT_CONFIG_DEFAULT=%~dp0..\config\cabum_config.txt
if exist "%~dp0..\..\..\..\my\cabum\cabum_config.txt" (
    set CABUM_BAT_CONFIG=%~dp0..\..\..\..\my\cabum\cabum_config.txt
) else (
    if exist "%~dp0..\..\config\cabum_config.txt" set CABUM_BAT_CONFIG=%~dp0..\..\config\cabum_config.txt
)
rem for /f "tokens=*" %%i in ("!CABUM_BAT_CONFIG!") do echo %%~fi
echo done

if "!CABUM_BAT_CONFIG_DEFAULT!"=="" (
	echo.|set /p = Default config: not found - using custom config only...
	echo done
) else (
	echo.|set /p =  Default config: 
	for /f "tokens=*" %%i in ("!CABUM_BAT_CONFIG_DEFAULT!") do echo %%~fi
)
if "!CABUM_BAT_CONFIG!"=="" (
	echo.|set /p = Custom config: not found - using SCHWINDT standard...
	echo done
) else (
	echo.|set /p =  Custom config: 
	for /f "tokens=*" %%i in ("!CABUM_BAT_CONFIG!") do echo %%~fi
)
rem case=0 - no config found
if "!CABUM_BAT_CONFIG!"=="" if "!CABUM_BAT_CONFIG_DEFAULT!"=="" set case=0
rem case=1 - only SCHWINDT config
if "!CABUM_BAT_CONFIG!"=="" if not "!CABUM_BAT_CONFIG_DEFAULT!"=="" set case=1
rem case=2 - only custom config
if not "!CABUM_BAT_CONFIG!"=="" if "!CABUM_BAT_CONFIG_DEFAULT!"=="" set case=2
rem case=3 - SCHWINDT and custom config
if not "!CABUM_BAT_CONFIG!"=="" if not "!CABUM_BAT_CONFIG_DEFAULT!"=="" set case=3

rem echo done
if defined CATREP_SERVER (
    set CATREP_SERVER_SOURCE=SYSTEM
) else (
    set CATREP_SERVER_SOURCE=CONFIG
)
if defined CATREP_REF (
    set CATREP_REF_SOURCE=SYSTEM
) else (
    set CATREP_REF_SOURCE=CONFIG
)

echo.|set /p =Reading config...
if "!case!"=="3" (
	if exist !temp!\cabum_config.txt.sample del !temp!\cabum_config.txt.sample
	set warning=yes
	set double_defined=no
	for /f "eol=# tokens=* usebackq" %%i in ("!CABUM_BAT_CONFIG_DEFAULT!") do (
		set apply=yes
		set temp1=%%i
		for /f "tokens=1 delims==" %%j in ("!temp1!") do (
			set var=%%j
			if "%%j"=="CATREP_SERVER" if "!CATREP_SERVER_SOURCE!"=="SYSTEM" set apply=no
			if "%%j"=="CATREP_SERVER" set warning=no
			if "%%j"=="CATREP_REF" if "!CATREP_REF_SOURCE!"=="SYSTEM" set apply=no
		)
		for /f "eol=# tokens=* usebackq" %%k in ("!CABUM_BAT_CONFIG!") do (		
			set temp2=%%k
			for /f "tokens=1 delims==" %%l in ("!temp2!") do (
				set my_var=%%l
				if "!my_var!"=="!var!" (
					if "!temp1!"=="!temp2!" (
						set double_defined=yes
					) else (
						set temp1=!temp2!
						echo !temp1! >> !temp!\cabum_config.txt.sample
					)
				)	
			)
		)
		if /i "!temp1:~0,3!"=="set" set temp1=!temp1:~4!
		if "!apply!"=="yes" (
			for /f "tokens=* delims= " %%n in ("!temp1!") do set temp1=set %%n
			!temp1!
		)
	)
	echo done
	echo. |set /p =Rereading custom config for own variables...
	for /f "eol=# tokens=* usebackq" %%k in ("!CABUM_BAT_CONFIG!") do (		
		for /f "tokens=1 delims==" %%l in ("%%k") do (
			set my_var=%%l
			if not defined !my_var! (
				set temp1=%%k
				if /i "!temp1:~0,3!"=="set" set temp1=!temp1:~4!
				for /f "tokens=* delims= " %%n in ("!temp1!") do set temp1=set %%n
				!temp1!
			)	
		)
	)
)
if "!case!"=="2" (
	for /f "eol=# tokens=* usebackq" %%i in ("!CABUM_BAT_CONFIG!") do (
		set apply=yes
		set temp1=%%i
		for /f "tokens=1 delims==" %%j in ("!temp1!") do (
			set var=%%j
			if "%%j"=="CATREP_SERVER" if "!CATREP_SERVER_SOURCE!"=="SYSTEM" set apply=no
			if "%%j"=="CATREP_REF" if "!CATREP_REF_SOURCE!"=="SYSTEM" set apply=no
		)
		if /i "!temp1:~0,3!"=="set" set temp1=!temp1:~4!
		if "!apply!"=="yes" (
			for /f "tokens=* delims= " %%n in ("!temp1!") do set temp1=set %%n
			!temp1!
		)
	)
)
if "!case!"=="1" (
	set warning=yes
	for /f "eol=# tokens=* usebackq" %%i in ("!CABUM_BAT_CONFIG_DEFAULT!") do (
		set apply=yes
		set temp1=%%i
		for /f "tokens=1 delims==" %%j in ("!temp1!") do (
			set var=%%j
			if "%%j"=="CATREP_SERVER" if "!CATREP_SERVER_SOURCE!"=="SYSTEM" set apply=no
			if "%%j"=="CATREP_SERVER" set warning=no
			if "%%j"=="CATREP_REF" if "!CATREP_REF_SOURCE!"=="SYSTEM" set apply=no
		)
		if /i "!temp1:~0,3!"=="set" set temp1=!temp1:~4!
		if "!apply!"=="yes" (
			for /f "tokens=* delims= " %%n in ("!temp1!") do set temp1=set %%n
			!temp1!
		)
	)
)
if "!case!"=="0" (
	echo No config found... Exiting... 
	goto :END_PAUSE
)
echo done
if "!warning!"=="yes" (
	echo.
	echo CATREP Server not defined in default config. Maybe old config?
	for /f "tokens=*" %%i in ("!CABUM_BAT_CONFIG_DEFAULT!") do echo Please check %%~fi
	ping 127.0.0.1 -n 10 > NUL
)
if "!double_defined!"=="yes" (
	echo.
	echo Some variables are equal in both configs. 
	echo Maybe old custom config used?  
	for /f "tokens=*" %%i in ("!CABUM_BAT_CONFIG!") do echo Please check %%~fi
	echo The file !temp!\cabum_config.txt.sample has been created. 
	echo There you can find the variables that differ from SCHWINDT standard.
	echo.
	ping 127.0.0.1 -n 10 > NUL
)


if "!CATREP_SERVER_SOURCE!"=="SYSTEM" echo Variable CATREP_SERVER set by system variable ...
if "!CATREP_SERVER_SOURCE!"=="CONFIG" echo Variable CATREP_SERVER set by config file ...
if "!CATREP_REF_SOURCE!"=="SYSTEM" echo Variable CATREP_REF set by system variable ...
if "!CATREP_REF_SOURCE!"=="CONFIG" echo Variable CATREP_REF set by config file ...

if not exist "!CATREP_REF!" (
	if not exist "!CABUM_HOME!" set CABUM_HOME=%~dp0..
	REM From highest to lowest priority
	if not exist "!CABUM_CONFIG!" set CABUM_CONFIG=%~dp0..\..\Config
	if not exist "!CABUM_CONFIG!" set CABUM_CONFIG=%~dp0..\Config
)
for /f "tokens=*" %%i in ("!CABUM_CONFIG!") do echo CABUM_CONFIG=%%~fi
echo.
if not exist "!CABUM_WORK!" mkdir "!CABUM_WORK!" 1>nul 2>&1

REM Detecting Windows-Version
set winxp64=no
set /p =Detecting Windows-Version...<nul
for /f "tokens=3" %%i in ('reg.exe query "HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion" /v "CurrentVersion" ^| find.exe /i "REG_SZ"') do (
	for /f "tokens=1,2 delims=." %%j in ("%%i") do (
		if "%%j"=="6" if "%%k"=="3" (
			set osversion=10
			echo Windows 10
		)
		if "%%j"=="6" if "%%k"=="2" (
			set osversion=8
			echo Windows 8
		)
		if "%%j"=="6" if "%%k"=="1" (
			set osversion=7
			echo Windows 7
		)
		if "%%j"=="6" if "%%k"=="0" (
			set osversion=6
			echo Windows Vista
		)
		if "%%j"=="5" if "%%k"=="2" (
			set osversion=5
			for /f "tokens=2*" %%i in ('reg.exe query "HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion" /v "ProductName" ^| "!find!" /i "REG_SZ"') do (
				if "%%j"=="Microsoft Windows XP" (
					echo Windows XP x64
					set winxp64=yes
				) else (
					echo Windows Server 2003
				)
			)
		)
		if "%%j"=="5" if "%%k"=="1" (
			set osversion=4
			echo Windows XP x86
		)
	)
)

REM Check for administrative rights
echo.|set /p =Checking for administrative rights...
set admin=no
bcdedit.exe 1>nul 2>&1
if not errorlevel 1 set admin=yes
echo !admin!

REM Detecting OS-Language
echo.|set /p =Detecting OS-Language...
help.exe| find.exe "Hilfe">nul
if errorlevel 1 (
	set locale=en
	echo English
) else (
	set locale=de
	echo German
)

REM Sprachanpassung fr Windows XP x64
if "!winxp64!"=="yes" set locale=en

echo.|set /p =Detecting CPU-architecture...
if "!PROCESSOR_ARCHITECTURE!"=="AMD64" (
	echo AMD64
	if exist "!JAVA64_HOME!" (
		set JAVA_HOME=!JAVA64_HOME!
	) else (
		reg.exe query "HKEY_LOCAL_MACHINE\SOFTWARE\JavaSoft\Java Runtime Environment" /v "CurrentVersion" 1>nul 2>&1
		if not errorlevel 1 (
			for /f "tokens=2*" %%i in ('reg.exe query "HKEY_LOCAL_MACHINE\SOFTWARE\JavaSoft\Java Runtime Environment" /v "CurrentVersion" ^| find.exe /i "REG_SZ"') do (
				set JAVA_CURRENT_VERSION=%%j
			)
		)
		reg.exe query "HKEY_LOCAL_MACHINE\SOFTWARE\JavaSoft\Java Runtime Environment\!JAVA_CURRENT_VERSION!" /v "JavaHome" 1>nul 2>&1
		if not errorlevel 1 (
			for /f "tokens=2*" %%i in ('reg.exe query "HKEY_LOCAL_MACHINE\SOFTWARE\JavaSoft\Java Runtime Environment\!JAVA_CURRENT_VERSION!" /v "JavaHome" ^| find.exe /i "REG_SZ"') do (
				set JAVA_HOME=%%j
			)
		)
		if "!JAVA_HOME!"=="" (
			reg.exe query "HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\JavaSoft\Java Runtime Environment" /v "CurrentVersion" 1>nul 2>&1
			if not errorlevel 1 (
				for /f "tokens=2*" %%i in ('reg.exe query "HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\JavaSoft\Java Runtime Environment" /v "CurrentVersion" ^| find.exe /i "REG_SZ"') do (
					set JAVA_CURRENT_VERSION=%%j
				)
			)
			reg.exe query "HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\JavaSoft\Java Runtime Environment\!JAVA_CURRENT_VERSION!" /v "JavaHome" 1>nul 2>&1
			if not errorlevel 1 (
				for /f "tokens=2*" %%i in ('reg.exe query "HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\JavaSoft\Java Runtime Environment\!JAVA_CURRENT_VERSION!" /v "JavaHome" ^| find.exe /i "REG_SZ"') do (
					set JAVA_HOME=%%j
				)
			)
		)
	)
)
if "!PROCESSOR_ARCHITECTURE!"=="x86" (
	echo X86
	echo.
	echo 32-bit OS is not supported...exiting...
	goto END_PAUSE
)
echo JAVA_HOME=!JAVA_HOME!

echo.|set /p =Checking perl...
if exist "!CATPRO_REF!\bin\perl\perl\bin\perl.exe" set perl_path_ref=!CATPRO_REF!\bin\perl\perl\site\bin;!CATPRO_REF!\bin\perl\perl\bin;!CATPRO_REF!\bin\perl\c\bin
if exist "!CATPRO_LOC!\bin\perl\perl\bin\perl.exe" set perl_path_loc=!CATPRO_LOC!\bin\perl\perl\site\bin;!CATPRO_LOC!\bin\perl\perl\bin;!CATPRO_LOC!\bin\perl\c\bin
if not "!perl_path_loc!"=="" (
	set path=!perl_path_loc!;!path!
)
if "!perl_path_loc!"=="" if not "!perl_path_ref!"=="" (
	set path=!perl_path_ref!;!path!
)
perl.exe -v 1>nul 2>&1
if errorlevel 1 (
	cls
	echo Perl could not be found, please install it manually!
	echo.
 	echo Cabum needs Perl to start CATIA. 
	echo No perl.exe found in PATH variable.
	goto END_PAUSE
) else (
	echo OK
)

echo.
echo ^(c^)2021 SCHWINDT DIGITAL GmbH
echo                  Callenberger Str. 8
echo                  D-96450 Coburg
echo                  Germany
echo                  Phone  +49 9561 5560 0
echo                  mailto info@schwindt.eu
echo.

set help=0
if "%1" == "-h" set help=1
if "%1" == "-help" set help=1
if "%1" == "/?" set help=1
if "%1" == "-?" set help=1
if "!help!" == "1" (
	echo Usage:
	echo   cabum.bat [-replicate -forcereplicate -debug]
	echo.
	echo Without parameters CABUM starts with user interface.
	echo.
	echo   -replicate         doesn't start GUI at the end. ^(usefull for replication only^)
	echo.
	echo   -forcereplicate    doesn't start GUI at the end. Deletes local "update.txt"
	echo                      before checking replication-status. Checks existence of
	echo                      CATREP-Server before deleting "update.txt". 
	echo                      ^(usefull for forced replication^)
	echo.
	echo   -debug             prints debug output to the console
	echo.                      
	goto END_PAUSE
)

echo.
echo -------------------------------------------------------------------------------
echo Check the configuration ...
echo -------------------------------------------------------------------------------
echo.

if not "!CATREP_SERVER!"=="" (
	dir "!CATREP_REF!" 1>nul 2>&1
	if not errorlevel 1 (
		echo Replication Server: !CATREP_SERVER!
		echo Reference CATSCT:   !CATSCT_REF!
		echo Reference CATMY:    !CATMY_REF!
		echo Reference CATPRO:   !CATPRO_REF!
		echo Reference CATCUS:   !CATCUS_REF!
		echo Reference FDEC:     !FDEC_REF!
		echo.
		echo Local replic:       !CATREP_LOC_3DX!
		echo Local replic ^(old^): !CATREP_LOC!
		echo.

		if "%1"=="-forcereplicate" (
			if exist "!CATSCT_LOC!\update.txt" del /f "!CATSCT_LOC!\update.txt"
		)

		if "!locale!"=="de" set acl_group=benutzer
		if "!locale!"=="en" set acl_group=users

		if not exist "!CATREP_LOC_3DX!" (
			echo Local replication directory !CATREP_LOC_3DX! does not exist, will create it ...
			mkdir "!CATREP_LOC_3DX!"
			if errorlevel 1 (
				echo *** Cannot create local replication directory !CATREP_LOC_3DX! ***
				echo Ask your admin to create directory !CATREP_LOC_3DX! with FULL ACCESS permissions!
				goto END_PAUSE
			)
			compact.exe /C /S:!CATREP_LOC_3DX! /A /I /Q 1>nul 2>&1
		)
		if not exist "!CATSCT_LOC!" (
			echo Local replication directory !CATSCT_LOC! does not exist, will create it ...
			mkdir "!CATSCT_LOC!"
			if errorlevel 1 (
				echo *** Cannot create local replication directory !CATSCT_LOC! ***
				echo Ask your admin to create directory !CATSCT_LOC! with FULL ACCESS permissions!
				goto END_PAUSE
			)
			cacls.exe "!CATSCT_LOC!" /T /E /G !acl_group!:F >nul
			compact.exe /C /S:!CATSCT_LOC! /A /I /Q 1>nul 2>&1
		)
		if not exist "!CATMY_LOC!" (
			echo Local replication directory !CATMY_LOC! does not exist, will create it ...
			mkdir "!CATMY_LOC!"
			if errorlevel 1 (
				echo *** Cannot create local replication directory !CATMY_LOC! ***
				echo Ask your admin to create directory !CATMY_LOC! with FULL ACCESS permissions!
				goto END_PAUSE
			)
			cacls.exe "!CATMY_LOC!" /T /E /G !acl_group!:F >nul
			compact.exe /C /S:!CATMY_LOC! /A /I /Q 1>nul 2>&1
		)
		if not "!FDEC_REF!"=="" if exist !FDEC_REF! (
			if not exist "!CATREP_LOC!" (
				echo Local replication directory !CATREP_LOC! does not exist, will create it ...
				mkdir "!CATREP_LOC!"
				if errorlevel 1 (
					echo *** Cannot create local replication directory !CATREP_LOC! ***
					echo Ask your admin to create directory !CATREP_LOC! with FULL ACCESS permissions!
					goto END_PAUSE
				)
			cacls.exe "!CATREP_LOC!" /T /E /G !acl_group!:F >nul
			compact.exe /C /S:!CATREP_LOC! /A /I /Q 1>nul 2>&1
			)
			if not exist "!CATCUS_LOC!" (
				echo Local replication directory !CATCUS_LOC! does not exist, will create it ...
				mkdir "!CATCUS_LOC!"
				if errorlevel 1 (
					echo *** Cannot create local replication directory !CATCUS_LOC! ***
					echo Ask your admin to create directory !CATCUS_LOC! with FULL ACCESS permissions!
					goto END_PAUSE
				)
			cacls.exe "!CATCUS_LOC!" /T /E /G !acl_group!:F >nul
			compact.exe /C /S:!CATCUS_LOC! /A /I /Q 1>nul 2>&1
			)
			if not exist "!FDEC_LOC!" (
				echo Local replication directory !FDEC_LOC! does not exist, will create it ...
				mkdir "!FDEC_LOC!"
				if errorlevel 1 (
					echo *** Cannot create local replication directory !FDEC_LOC! ***
					echo Ask your admin to create directory !FDEC_LOC! with FULL ACCESS permissions!
					goto END_PAUSE
				)
			cacls.exe "!FDEC_LOC!" /T /E /G !acl_group!:F >nul
			compact.exe /C /S:!FDEC_LOC! /A /I /Q 1>nul 2>&1
			)
		)
		echo.
		echo -------------------------------------------------------------------------------
		echo Check the replication status ...
		echo -------------------------------------------------------------------------------
		echo.
		set UPDATE_NEED=NO
		set UPDATE_FORCE=NO
		set UPDATE_ASK=NO
		set UPDATE_CANCEL=NO
		echo Check !CATREP_REF!\update.txt ...
		echo.
		if not exist "!CATREP_REF!\update.txt" (
			echo ERROR: !CATREP_REF!\update.txt does not exist, will exit ...
			goto END_PAUSE
		)
		fc.exe "!CATREP_REF!\update.txt" "!CATSCT_LOC!\update.txt" 1>nul 2>&1
		if errorlevel 1 ( 
			set UPDATE_NEED=YES
			set UPDATE_FORCE=YES
		)
		echo Check !CATREP_REF!\update_soft.txt ...
		echo.
		fc.exe "!CATREP_REF!\update_soft.txt" "!CATSCT_LOC!\update_soft.txt" 1>nul 2>&1
		if errorlevel 1 if exist "!CATREP_REF!\update_soft.txt" ( 
			set UPDATE_NEED=YES
			set UPDATE_ASK=YES
		)
        REM wla, 13.06.13: support for slave_in_update.txt (MASTER to SLAVE replication)
        REM   if this file is present, our CATREP is a slave SERVER in replication right now
        REM   and we skip local replication until the file disappear ...
        IF EXIST "%CATREP_REF%\slave_in_update.txt" (
          ECHO.
          ECHO ***   %CATREP_REF%\slave_in_update.txt detected   ***
          ECHO *** It seems that your CATREP is updated from a MASTER right now ***
          ECHO If this message persist for more then 1 hour, please inform your
          ECHO administrator.
          ECHO Skip local replication tasks ...
          ECHO.
          REM sleep for 20s
          ping.exe -n 20 127.0.0.1 > NUL
          set UPDATE_NEED=NO
        )
		set path_backup=!path!
		if "!UPDATE_NEED!"=="YES" (
			if not "!perl_path_ref!"=="" (
				set path=!perl_path_ref!;!path!
			)
			set count=0
			for /f "tokens=*" %%i in ('where perl.exe') do if "!count!"=="0" (
				echo Perl: %%i
				set count=1
			)
			for /f "tokens=9,12 delims=^(^) " %%i in ('perl.exe -v ^| find.exe /i "This is perl"') do (
				echo       %%i ^(%%j^)
			)
			echo.
			echo *******************************************************************************
			echo ***   Your CATIA admin has planned an update for the CATIA customization    ***
			echo *******************************************************************************
			echo.
			REM Posibility to bypass temporary the update
			if "!UPDATE_ASK!"=="YES" if "!UPDATE_FORCE!"=="NO" (
				for /f "tokens=*" %%i in (!CATREP_REF!\update_soft.txt) do (
					set message=%%i
				)
				echo You can choose to bypass the update. If you do so you will be 
				echo asked again at next start.
				echo.
				echo ***       Please accept the update installation as soon as possible        ***
				if not "!message!" == "" ( 
					echo.
					echo Reason: !message!
				)
				echo.
				set /P CHOICE=Install it now? [y]es or [n]o: 
				if /I "!CHOICE!"=="n" set UPDATE_CANCEL=YES
			)
			if "!UPDATE_CANCEL!"=="NO" (
				echo.
				echo Replication to !CATREP_LOC_3DX!, please be patient ...
				REM Delete trigger file, for forced update in case of interrupt the update
				del "!CATSCT_LOC!\update.txt" 1>nul 2>&1
				perl.exe "!CATPRO_REF!\cabum\bin\sctrep.pl"
				xcopy.exe "!CATREP_REF!\update.txt" "!CATSCT_LOC!" /Q /H /Y 1>nul 2>&1
				xcopy.exe "!CATREP_REF!\update_soft.txt" "!CATSCT_LOC!" /Q /H /Y 1>nul 2>&1
				echo Replication finished ...
				echo.
				set /p =Restarting cabum.bat in 5 seconds...<nul
				ping.exe -n 2 127.0.0.1 > nul
				set /P =<nul
				set /p =4 seconds...<nul
				ping.exe -n 2 127.0.0.1 > nul
				set /P =<nul
				set /p =3 seconds...<nul
				ping.exe -n 2 127.0.0.1 > nul
				set /P =<nul
				set /p =2 seconds...<nul
				ping.exe -n 2 127.0.0.1 > nul
				set /P =<nul
				set /p =1 seconds...<nul
				ping.exe -n 2 127.0.0.1 > nul
				set /P =<nul
				set /p =0 seconds...<nul
				ping.exe -n 2 127.0.0.1 > nul
				cls
				set path=!path_backup!
				if "%1" == "-forcereplicate" !CABUM_HOME!\bin\cabum.bat -replicate
				"!CABUM_HOME!\bin\cabum.bat" %1
			)
		) else (
			echo Replication to !CATREP_LOC! not necessary ...
		)
	) else (
		echo Replication server !CATREP_SERVER! not available, skip replication ...
	)
) else (
  echo No replication server defined, will run without ...
)

echo.
set count=0
for /f "tokens=*" %%i in ('where perl.exe') do if "!count!"=="0" (
	echo Perl: %%i
	set count=1
)
for /f "tokens=9,12 delims=^(^) " %%i in ('perl.exe -v ^| find.exe /i "This is perl"') do (
	echo       %%i ^(%%j^)
)

echo.
REM testing CATUserSettings-prereq
if not defined CATUSERSETTINGS_ROOT (
    echo Variable "CATUSERSETTINGS_ROOT" not defined, using fallback ...
    set CATUSERSETTINGS_ROOT=!APPDATA!\DassaultSystemes
) else (
    dir "!CATUSERSETTINGS_ROOT!" 1>nul 2>&1
    if errorlevel 1 (
        echo !CATUSERSETTINGS_ROOT! not reachable, using fallback ...
        set CATUSERSETTINGS_ROOT=!APPDATA!\DassaultSystemes
    )
)
echo CATUSERSETTINGS_ROOT:  !CATUSERSETTINGS_ROOT!

if not exist "!JAVA_HOME!\Bin" (
	echo Check JAVA_HOME [!JAVA_HOME!], will exit ...
	goto END_PAUSE
)
if not exist "!CABUM_HOME!" (
	echo Check CABUM_HOME [!CABUM_HOME!], will exit ...
	goto END_PAUSE
)
if not exist "!CABUM_CONFIG!" (
	echo Check CABUM_CONFIG [!CABUM_CONFIG!], will exit ...
	goto END_PAUSE
)

REM set PATH with local extension, avoid repeated extension
if not "!CABUM_PATH_EXTENSION!" == "YES" (
  set PATH=!CABUM_HOME!\Bin;!CATPRO_LOC!\Bin;!JAVA_HOME!\Bin;!PATH!
  set CABUM_PATH_EXTENSION=YES
)

if "!locale!"=="en" set programme=programs
if "!locale!"=="de" set programme=programme

if !osversion! LEQ 5 (
	set alluserdesktop=!allusersprofile!\desktop
	set alluserstartmenu=!allusersprofile!\startm~1\!programme!
	set userdesktop=!userprofile!\desktop
	set userstartmenu=!userprofile!\startm~1\!programme!
) else (
	set alluserdesktop=!public!\desktop
	set alluserstartmenu=!allusersprofile!\Microsoft\Windows\Start Menu\Programs
	set userdesktop=!userprofile!\desktop
	set userstartmenu=!userprofile!\AppData\Roaming\Microsoft\Windows\Start Menu\Programs
)

REM Copy Link to Desktop/Startmenu
if exist "!temp!\CABUM.lnk" del /f /q "!temp!\CABUM.lnk" 1>nul 2>&1
cscript.exe //nologo "!CABUM_HOME!\bin\mklink.vbs" /source:"!temp!\CABUM.lnk" /target:"!CABUM_HOME!\bin\cabum.bat" /symbol:"!CABUM_HOME!\img\cabum.ico" /hotkey:"Alt+Ctrl+1"
if "!admin!"=="yes" (
	if not exist "!alluserdesktop!\CABUM.lnk" (
		if !CABUM_ALLUSERDESKTOP_LINK! geq 1 copy /y "!temp!\CABUM.lnk" "!alluserdesktop!" 1>nul 2>&1
	)
	if not exist "!alluserstartmenu!\CABUM.lnk" (
		if !CABUM_ALLUSERSTARTMENU_LINK! geq 1 copy /y "!temp!\CABUM.lnk" "!alluserstartmenu!" 1>nul 2>&1
	)
	if exist "!allusersprofile!\startm~1\CABUM.lnk" del /f /q "!allusersprofile!\startm~1\CABUM.lnk"
	if exist "!allusersprofile!\startm~1\CABUM6.lnk" del /f /q "!allusersprofile!\startm~1\CABUM6.lnk"
	if exist "!alluserdesktop!\CABUM6.lnk" del /f /q "!alluserdesktop!\CABUM6.lnk"
	if exist "!alluserstartmenu!\CABUM6.lnk" del /f /q "!alluserstartmenu!\CABUM6.lnk"
)
if "!admin!"=="no" (
	if not exist "!alluserdesktop!\CABUM.lnk" if not exist "!userdesktop!\CABUM.lnk" (
		if !CABUM_USERDESKTOP_LINK! geq 1 copy /y "!temp!\CABUM.lnk" "!userdesktop!" 1>nul 2>&1
	)
	if not exist "!alluserstartmenu!\CABUM.lnk" if not exist "!userstartmenu!\CABUM.lnk" (
		if !CABUM_USERSTARTMENU_LINK! geq 1 copy /y "!temp!\CABUM.lnk" "!userstartmenu!" 1>nul 2>&1
	)
	if exist "!userdesktop!\CABUM6.lnk" del /f /q "!userdesktop!\CABUM6.lnk"
	if exist "!userstartmenu!\CABUM6.lnk" del /f /q "!userstartmenu!\CABUM6.lnk"
)
if exist "!alluserdesktop!\CABUM.lnk" if exist "!userdesktop!\CABUM.lnk" del /f /q "!userdesktop!\CABUM.lnk" 1>nul 2>&1
if exist "!alluserstartmenu!\CABUM.lnk" if exist "!userstartmenu!\CABUM.lnk" del /f /q "!userstartmenu!\CABUM.lnk" 1>nul 2>&1
if exist "!userprofile!\startm~1\CABUM.lnk" del /f /q "!userprofile!\startm~1\CABUM.lnk" 1>nul 2>&1
if exist "!userprofile!\startm~1\CABUM6.lnk" del /f /q "!userprofile!\startm~1\CABUM6.lnk" 1>nul 2>&1
if exist "!userprofile!\AppData\Roaming\Microsoft\Windows\Start Menu\CABUM6.lnk" del /f /q "!userprofile!\AppData\Roaming\Microsoft\Windows\Start Menu\CABUM6.lnk" 1>nul 2>&1

if exist "!temp!\CABUM.lnk" del /f /q "!temp!\CABUM.lnk" 1>nul 2>&1

if not exist "!CABUM_WORK!" mkdir "!CABUM_WORK!"

REM Check and execute second customer batch
if exist "%~dp0..\..\config\cabum-pre-customer.bat" set custscript2=%~dp0..\..\config\cabum-pre-customer.bat
if exist "%~dp0..\..\..\..\..\catia\sct\fdec\site\all\cabum\cabum-pre-customer.bat" set custscript2=%~dp0..\..\..\..\..\catia\sct\fdec\site\all\cabum\cabum-pre-customer.bat
if exist "%~dp0..\..\..\..\my\cabum\cabum-pre-customer.bat" set custscript2=%~dp0..\..\..\..\my\cabum\cabum-pre-customer.bat
if exist "!custscript2!" (
	echo.
	echo Execute customer second batch file "!custscript2!"
	echo.
	call "!custscript2!"
)

if exist "!CATPRO_LOC!\bin\jre" (
	for /f "tokens=2 delims==" %%i in ('type "!CATPRO_LOC!\bin\jre\release" ^| findstr.exe /i /r /c:"\<JAVA_VERSION\>"') do (
		set java_version=%%i
		set java_version=!java_version:"=!
		if "!java_version:~0,2!"=="11" (
			echo Java Version ^(CABUM^): !java_version!
			set JAVA11_HOME=!CATPRO_LOC!\bin\jre
		) else (
			echo Java Version ^(CABUM^): !java_version!
			echo.
			echo Java Version not compatible with CABUM...exiting...
			goto END_PAUSE
		)
	)
) else (
	reg.exe query "HKEY_LOCAL_MACHINE\SOFTWARE\AdoptOpenJDK\JRE"  1>nul 2>&1
	if not errorlevel 1 (
		for /f "tokens=5 delims=\" %%i in ('reg.exe query "HKEY_LOCAL_MACHINE\SOFTWARE\AdoptOpenJDK\JRE"') do (
			set JAVA11_CURRENT_VERSION=%%i
		)
	)
	reg.exe query "HKEY_LOCAL_MACHINE\SOFTWARE\AdoptOpenJDK\JRE\!JAVA11_CURRENT_VERSION!\openj9\MSI" /v "Path" 1>nul 2>&1
	if not errorlevel 1 (
		for /f "tokens=2*" %%i in ('reg.exe query "HKEY_LOCAL_MACHINE\SOFTWARE\AdoptOpenJDK\JRE\!JAVA11_CURRENT_VERSION!\openj9\MSI" /v "Path" ^| find.exe /i "REG_SZ"') do (
			for /f "tokens=2 delims==" %%i in ('type "%%j\release" ^| findstr.exe /i /r /c:"\<JAVA_VERSION\>"') do (
				set java_version=%%i
				set java_version=!java_version:"=!
				if "!java_version:~0,2!"=="11" (
					echo Java Version ^(CABUM^): !java_version!
					set JAVA11_HOME=%%j
				) else (
					echo Java Version ^(CABUM^): !java_version!
					echo.
					echo Java Version not compatible with CABUM...exiting...
					goto END_PAUSE
				)
			)
		)
	)
)

if "!JAVA11_HOME!"=="" (
	echo CABUM needs Java 11 to run.
	echo Java 11 is not installed/found on this computer.
	echo Please install it...will exit now...
	goto END_PAUSE
)

set CLASSPATH=!CABUM_HOME!\Java\cabum.jar

for /f "tokens=*" %%i in ('dir /b /a-d /on "!CABUM_HOME!\Java\lib\*.jar"') do (
	set CLASSPATH=!CLASSPATH!;!CABUM_HOME!\Java\lib\%%i
)

REM a dump of the environment may be nice
set > "%USERPROFILE%\.cabum\set.txt"

if "%1" == "" (
	start "CABUM" "!JAVA11_HOME!\bin\javaw.exe" -Djava.class.path="!CLASSPATH!" -Dcabum.home="!CABUM_HOME!" -Dcabum.plugin="!PLUGIN_DIR!" -Dcabum.config="!CABUM_CONFIG!" -Dcabum.config_loc="!CABUM_CONFIG_LOC!" -Dcabum.work="!CABUM_WORK!" -Dcabum.lang="!CABUM_LANG!" -Duser.appdata="!APPDATA!" -Dalluser.home="!ALLUSERSPROFILE!" -Duser.home="!USERPROFILE!" net.schwindt.cabum.framework.ui.CL_Frame_Cabum
	goto END
)
if "%1" == "-debug" (
	start "CABUM" "!JAVA11_HOME!\bin\javaw.exe" -Djava.class.path="!CLASSPATH!" -Dcabum.home="!CABUM_HOME!" -Dcabum.plugin="!PLUGIN_DIR!" -Dcabum.config="!CABUM_CONFIG!" -Dcabum.config_loc="!CABUM_CONFIG_LOC!" -Dcabum.work="!CABUM_WORK!" -Dcabum.lang="!CABUM_LANG!" -Duser.appdata="!APPDATA!" -Dalluser.home="!ALLUSERSPROFILE!" net.schwindt.cabum.framework.ui.CL_Frame_Cabum -debug
	goto END
)
if "%1" == "-replicate" goto END
if "%1" == "-forcereplicate" goto END

:END_PAUSE
echo.
echo Press a key to exit ...
pause > nul
goto EXIT

:END
REM sleep for 5s
ping.exe -n 6 127.0.0.1 > nul

:EXIT
endlocal
rem exit
