#!/usr/bin/perl
##############################################################################
# Owner : Walter Laub (walter.laub@schwindt.eu)                              #
#                                                                            #
# Name      : sctcatstart.pl                                                 #
# Intention : like original from D.S., but plattform independent, with       #
#             improvements and more features                                 #
# Require   : perl                                                           #
#                                                                            #
# 28.02.05 wla : created                                                     #
# v1.01, 28.06.05 : support for aix_a64 introduced with V5R15                #
#		            print return code                                        #
#		            new options: nocaa, noscripts, nodlnames, nohotfix, novar#
# v1.02, 18.07.05 : check if spacemouse already running                      #
# v1.03, 25.08.05 : explicit set the display to hostname:0.0, without (:0.0) #
#                   CATIA abort by some operations with V5MEM=8              #
# v1.03a,01.10.05 : minor bug repair: evaluation of caa version to start     #
# v1.04,23.11.05  : bug repair: automatically calculation of V5MEM must      #
#                   return an integer                                        #
# v1.05,21.12.05  : variable LDR_CNTRL changed with V5R16 for aix_a64        #
# v1.06,07.02.06  : -spctty now also accept the real device name             #
#                   V5CAA now scan all subdirectory for perl scripts         #
# v1.06a,01.03.06 : bug repair: error, when $home/v5dlnames not present      #
# v1.06b,02.03.06 : bug repair: error, when $home/v5dlnames not present      #
# v1.06c,20.04.06 : bug repair: not all V5CAA were found                     #
# v1.06d,06.06.06 : bug repair: $^0 for SunOS is "solaris"                   #
# v1.07,21.07.06  : now we support win_b64                                   #
# v1.07a,21.07.06 : the initial 1.07 has a bug with unix OSDS                #
# v1.08,07.11.06  : set again OSDS on Windows, we need it for caa-path       #
#                   recognize ostype for Windows 64 Bit                      #
# v1.09           : bug repair: infinite loop problem,                       #
#                               when evaluating Windows variables            #
#                   first VISTA changes                                      #
# v1.10           : implementing stop_*.pl scripts                           #
# v1.11           : shows the installation path in titlebar                  #
# v1.12           : delete the DLNames.CATSettings before import - makes     #
#                   trouble if the file exists and no entries in             #
#                   v5dlnames.txt                                            #
# v1.13           : version 1.13 omitted ;-)                                 #
# v1.14           : now the titlebar (via macro) is customizable             #
# v1.15,30.04.09  : changed titlebar (no macro)                              #
#                   $USERPROFILE/.catia as root for all extra needed         #
#                   directories                                              #
# v1.16,27.05.09  : the changed titlebar does not work with whitespaces in   #
#                   CATIA installation path, because of the glob operator    #
#                   replaced globally glob with bsd_glob resolve this issue  #
# v1.17,09.06.09  : check and exit if CABUMInstaller 2.x works on            #
#                   CATInstallPath                                           #
# v1.18,10.06.09  : check for required perl version                          #
#                   (since bsd_glob we need this)                            #
# v1.19,08.12.09  : problem under unix when evaluating variables like        #
#                   ${var1}${var2} without caracter between                  #
# v1.20,29.01.10  : extend also the CATKnowledgePath                         #
# v1.21,25.05.11  : defined(@array) is deprecated                            #
# v1.22,17.04.12  : changed detection of release, servicepack, hotfix        #
#                   for titlebar purpose (needed for V6 and V5R22)           #
# v1.30,07.09.12  : support for Siemens NX                                   #
#                   support for suppress and redirect output ($CABUMCONSOLE) #
# v1.30a,16.01.13 : path of titlebar.conf in the new installation schema     #
# v1.30b,23.01.13 : dump of environment variables to .cabum .catia .nx       #
# v1.31,06.02.13  : some changes for -lic, $ENV{MYLIC}                       #
# v1.32,05.07.13  : Variable CATLOWPRIORITY=1/NXLOWPRIORITY=1 starts         #
#                   CATIA/NX process with lowest priority                    #
# v1.33a,25.07.13 : some improvements to -p (projects)                       #
# v1.34,04.12.13  : support for NX UGII_USER_PROFILE_DIR                     #
# v1.35,08.01.14  : support for ICMLicense.CATSettings                       #
#                  (in same folder as LicensingCATSettings)                  #
# v1.36,07.08.14  : correct Hotfix detection for titlebar                    #
# v1.37,26.02.15  : print all command-line arguments                         #
#                   set some variables, to work when bypassing CABUM         #
# v1.38,07.08.15  : support for license by writing and import a xml-file     #
# v1.38b,07.08.15 : a CABUM selected license can be displayed in titlebar    #
# v1.39,05.11.15  : a "$" in directory names from PATH variable on windows   #
#                   cause issues in eval_variables; now disabled for windows #
# v1.40,06.10.16  : set an empty UGII_UGDOC_BASE to siemens online help      #
# v1.41,14.09.17  : shows the project name in titlebar                       #
# v1.42,12.02.18  : lock selected and all other licenses                     #
#                   search in well known directories for environments        #
#                     c:\3dx\[schwindt|my]\[catia|nx]\*\*\env\*              #
#                   set CATUSERSETTINGS_ROOT if not set                      #
# v1.43,12.04.18  : also lock addon-configurations from optional myPCFG.list #
# v1.43a,18.04.18 : output of SPLM_LICENSE_SERVER and                        #
#                             HDDRUCK_LICENSE_FILE before start NX           #
# v1.44,06.06.18  : set return code as $ENV{CADRETURNCODE}                   #
# v1.45,15.06.18  : append "-e 3DEXPRunOperation" to CNEXT.exe and DMU.exe   #
#                   needed by 3dexperience 2018x to load objects             #
#                   from browser to V5 session                               #
# v1.46,09.07.19  : glob operator with issues on UNC path                    #
#                   replaced with File::Glob::Windows                        #
#                   replacing the module inherit issues with "$_"            #
# v1.47,29.11.19  : environment variable No3DEXPRunOperation disable         #
#                   (default) appending "-e 3DEXPRunOperation"               #
# v1.48,02.01.20  : replaced function glob with "*" with find                #
# v1.48a,02.01.20 : ^start_*.pl not start_*.pl                               #
# v1.49,06.03.20  : clean AUTOREF on every start                             #
# v1.50,06.03.20  : add optional environment and project variables in MY     #
# v1.51,09.03.20  : increase compatibility to GUI for optional environment   #
#                   and project variables in MY                              #
# v1.51a,09.03.20 : optional environment and project variables in MY - the   #
#                   extension ist "add.txt" not "txt.add" as in v1.51        #
# v1.52,08.05.20  : DLNAMES overwriting, last wins                           #
# v1.52a,08.05.20 : also dump variable UGS_LICENSE_BUNDLE to output          #
##############################################################################
#                                                                            # 
my $MyVersion="1.52a";
my $RequiredPerlVersion=5.024001;
#                                                                            #
##############################################################################

use strict;
use File::Basename;
use File::Copy;
use File::Spec::Functions;
use File::Path;
use File::Glob ':glob';
use File::Find;
use Cwd;
use Getopt::Long;
use Sys::Hostname;
 
my ($ostype, $plattform, $cmdopt, $apppath, $nxpath, $home, $hostname, $logname, $log, $date_s, $time_s, $timestamp_s, $app, $temp, $dummy, $ret, $os_var_like_ds);
my $official_program_name;
my $remotearg=""; my $enovoptions="";
my %opt=();
my @v5project=(); my @catref=();
my ($environment_directory, $environment_name, $environment_name_add, $project_add, $ENVDIR_);
my $mywork;					# $home/.catia, a working dir for user
my $myenvwork;				# $mywork/[ENVNAME], a working dir for user and env
my $myautoref;				# $myenvwork/autoref, referencies for user and env
my $mytitlebar;				# $myenvwork/titlebar, titlebar
my $os_var_like_ds="no";	# [yes|no]
my $debug="no";				# [yes|no]
my $seperator;

# let discover our os
for ($^O) {
  /aix/i && do {
    $ostype='unix'; $plattform='AIX';   last; };
  /hp-?ux/i && do {
    $ostype='unix'; $plattform='HP-UX'; last; };
  /irix.*/i && do {
    $ostype='unix'; $plattform='IRIX';  last; };
  /sunos|solaris/i && do {
    $ostype='unix'; $plattform='SunOS'; last; };
  /M*S*Win/i && do {
    $ostype='win';  $plattform='WIN';   last };
  # OTHER
  &print_error("Plattform $^ is not supported");
  exit 1;
}
if ($plattform eq 'IRIX' || $plattform eq 'SunOS') { 
  system 'alias -x awk=nawk'; # like "catstart"
}

if ($ostype eq 'win') {
  require Win32;
  eval "require Win32::OLE";
  eval "import Win32::OLE qw ( in )";
  eval "require Win32::Process";
  eval "require File::Glob::Windows";
  eval "use File::Glob::Windows";
  if($@) {
    print "\n!!! Module File::Glob::Windows not installed !!!\n";
    print "This results in unpredictable behavior with UNC paths!\n";
    print "Please check your Perl installation!\n\n";
  }    
  $logname=$ENV{USERNAME};
  $home=$ENV{USERPROFILE};
  $temp=$ENV{TEMP};
  $temp=$ENV{TMP} if ! defined $temp;
  $seperator=";";
  $ENVDIR_=catfile("c:","3dx","schwindt") . " " . catfile("c:","3dx","my") . " " . catfile("c:","catia","sct","cus5","env") . " " . catfile("c:","catia","sct","fdec","env");	# SCT
  # for compatibility, when bypass CABUM we set the following variables
  $ENV{CATSCT_LOC}='c:\3dx\schwindt'             if (! defined $ENV{CATSCT_LOC} && -d 'c:\3dx\schwindt');
  $ENV{CATSCT_LOC_CATIA}='c:\3dx\schwindt\catia' if (! defined $ENV{CATSCT_LOC_CATIA} && -d 'c:\3dx\schwindt\catia');
  $ENV{CATSCT_LOC_NX}='c:\3dx\schwindt\nx'       if (! defined $ENV{CATSCT_LOC_NX} && -d 'c:\3dx\schwindt\nx');
  $ENV{CATMY_LOC}='c:\3dx\my'                    if (! defined $ENV{CATMY_LOC} && -d 'c:\3dx\my');
  $ENV{CATMY_LOC_CATIA}='c:\3dx\my\catia'        if (! defined $ENV{CATMY_LOC_CATIA} && -d 'c:\3dx\my\catia');
  $ENV{CATMY_LOC_NX}='c:\3dx\my\nx'              if (! defined $ENV{CATMY_LOC_NX} && -d 'c:\3dx\my\nx');
}
else {	# unix
  $logname=$ENV{LOGNAME};
  $logname=$ENV{USER} if ! defined $logname;
  $home=$ENV{HOME};
  $temp=$ENV{TEMP};
  $temp="/tmp" if ! defined $temp;
  $seperator=":";
  $ENVDIR_=catfile("/","catcus5","env") . " " . catfile("/","catcus5","fdec","env");	# SCT
}

my $MyName=basename $0;
$ENV{Sys}=$plattform;	# like "catstart"
$ENV{PLATTFORM}=$plattform;
$ENV{OSTYPE}=$ostype;
$hostname=hostname;
$debug=undef if ! ($debug=~/yes/i);

$timestamp_s=&getTimeStamp;
($date_s, $time_s)=&getDateAndTime; # get date and time string

# console or log to file?
$log=catfile($home, ".cabum", "sctcatstart_" . $timestamp_s . ".log");
mkpath(dirname($log),0) if ! -d dirname($log);
# delete log files older 1 day
foreach ( glob( catfile(dirname($log), "sctcatstart_*.log")) ) {
    unlink $_ if (-f && 1 < -M $_);
}
if ( "$ENV{CABUMCONSOLE}" eq "0" || "$ENV{CABUM_CONSOLE}" eq "0" ) {
  close STDOUT; close STDERR;
  open(STDOUT, ">>$log");
  open(STDERR, ">>$log");
  Win32::SetChildShowWindow(0) if defined &Win32::SetChildShowWindow; 
}

# for faster remote access, we deactivate some CATIA features
if ( $ostype eq 'unix' && ! ($ENV{DISPLAY} =~ /:0/) ) {
  $ENV{CNEXTSPLASHSCREEN}='No'; # no D.S. movie
  $ENV{CNEXTBACKGROUND}='No';   # no star wars
}

# for compatibility running outside CABUM, we check and set CATUSERSETTINGS_ROOT
$ENV{CATUSERSETTINGS_ROOT}=catfile($ENV{APPDATA}, "DassaultSystemes") if ! defined($ENV{CATUSERSETTINGS_ROOT});

# remove obsolete directories v5autoref, v5dlnames, v5macros in $home (v1.15)
rmtree (catfile($home, "v5autoref")) if -d (catfile($home, "v5autoref"));
rmtree (catfile($home, "v5dlnames")) if -d (catfile($home, "v5dlnames"));
rmtree (catfile($home, "v5macros"))  if -d (catfile($home, "v5macros"));

if ( defined($ENV{OLD_ENVIRONMENT_MODE}) ) {
  &print_warn("!!! Sorry, $MyName currently does not support OLD_ENVIRONMENT_MODE !!!");
  sleep 10;
  &print_h2("Will try to start without ...!");
}

# save the cmd options for debug output
$cmdopt=(join " ", @ARGV);
  
# assign input parameters from command line
&GetOptions(\%opt, 'h|help',
                   'env=s',
                   'direnv|d|envdir=s',
                   'senv=s',
                   'sdirenv=s',
                   'object=s',
                   'run=s',
                   'remote=s',
                   's',
                   'serv:s',
                   'port:i',
                   'user:s',
# additionally options, not supported by origibal script from DS		   
                   'project|p:s',
                   'catref:s',
                   'reset',
                   'lic:s',
                   'spc:s',
                   'spctty:s',
                   'nocaa',
                   'noscripts',
                   'nodlnames',
                   'nohotfix',
                   'novar',
)
or &printusage and exit;

# check if help needed
if ( defined $opt{h} ) {
  &printusage;
  exit;
}

&print_schwindt;
&check_required_perl;
&print_h1("Check options ...");

# Check the options
if ( ! defined($opt{env}) ) {
  # environment-name not specified
  &print_error("No environment-name specified!");
  exit 1;
}
if ( defined($opt{direnv}) ) {
  # environment-directory specified
  if ( ! -d $opt{direnv} ) {
    # environment-directory does not exist
    &print_error("Cannot find environment-directory \"$opt{direnv}\"");
    exit 1;
  }
  else {
    $environment_directory=$opt{direnv};
    $environment_name=catfile($environment_directory, $opt{env});
    if ( -e ($environment_name . ".txt") ) {
      $environment_name=$environment_name . ".txt";
    }
  }
}
if ( ! defined($opt{direnv})) {
  my @v5envdirs; my $knowndirs=0; my $v5envdir;
  # environment-directory not specified
  &print_info("'-direnv environment-directory' not specified");
  if ( defined($ENV{V5ENVDIR}) ) {
    &print_h2("Test shell variable V5ENVDIR, found V5ENVDIR=\"$ENV{V5ENVDIR}\"");
    @v5envdirs=split / /, $ENV{V5ENVDIR};
  }
  else {
    &print_h2("Test shell variable V5ENVDIR, not set");
    if ( defined($ENVDIR_) ) {
      $knowndirs=1;
      &print_h2("Test for well-known directories \"$ENVDIR_\"");
      @v5envdirs=split / /, $ENVDIR_;
    }
  }
  foreach (@v5envdirs) {
    $v5envdir=$_;
    if ( ! -d $v5envdir ) {
      if ( defined($knowndirs) ) {
        &print_h2("\"$v5envdir\" not found",3);
      }
      else {
        &print_error("Cannot find environment-directory \"$v5envdir\"");
        exit 1 if ! defined($knowndirs);
      }
    }
    $environment_name=catfile($v5envdir, $opt{env});
    if ( -e ($environment_name . ".txt") ) {
      $environment_name=$environment_name . ".txt";
    }
    if ( -e $environment_name ) {
      $opt{direnv}=$v5envdir;
      last;
    }
	# search in subdirecories
    foreach ( glob (catfile($v5envdir, "/*/*/*/env/", $opt{env} . ".txt") ) ) {
	  $environment_name=$_;
      $opt{direnv}=$_;
	  last;
	}
    foreach ( glob (catfile($v5envdir, "/*/*/*/env/", $opt{env}) ) ) {
	  $environment_name=$_;
      $opt{direnv}=$_;
	  last;
	}
	if ( -e $environment_name ) {
	  last;
	}
  }
}
if ( ! -e $environment_name ) {
  &print_error("Cannot find environment-file \"$opt{env}\[.txt\]\"");
  exit 1;
}
if ( ! -r $environment_name ) {
  &print_error("The environment-file \"$environment_name\[.txt\]\" is not readable");
  exit 1;
}
# license with old behaviour and new
# -lic with priority
$ENV{MYLIC}=$opt{lic} if defined $opt{lic};
$opt{lic}=$ENV{MYLIC};

# unix: set and fix the DISPLAY
if ( $ostype eq 'unix' ) {
  if ( $ENV{DISPLAY} =~ /^(:[0-9].*)/ ) {
    $ENV{DISPLAY}=$hostname . $1
  }
  if ( ! defined($ENV{DISPLAY}) || $ENV{DISPLAY} eq '' ) {
    $ENV{DISPLAY}=$hostname . ":0.0";
  }
}
if ($ostype eq 'unix') {
  &print_h1("Run GetOSDS ...");
  &getosds;		# set shellvariable OSDS, like "catstart"
}

# Source and detect the type of the environment
&print_h1 ("Source and detect type of the environment $environment_name ...");
$app=&source_env($environment_name);		# source the environment
print "  $app\n";

# check programname to start
if ( ! defined($opt{run}) && ! defined($opt{reset}) ) {
  # program_name not specified
  $opt{run}='CNEXT.exe' if $app eq "catia";
  $opt{run}='UGRAF.exe' if $app eq "nx";
  &print_info("'-run program_name' not specified, will start with '-run \"$opt{run}\"'");
}

# Only CATIA will set later a new official_program_name
$official_program_name=$opt{run};
if ( $app eq "catia" ) {
  $ENV{CATEnvName}=$environment_name;	    # set CATEnvName, like catstart
  $_=$environment_name;
  $environment_directory=dirname($environment_name);
  $ENV{CATEnvDir}=$environment_directory; # set CATEnvDir, like catstart
  $_=basename($environment_name);
  s/\.[^\.]*$//;			# eliminate extension
  $ENV{V5ENV}=$_;
  $ENV{V5ENVDIR}=$environment_directory;

  $apppath=&getCatiaPath;
  print "  CATIAPATH: $apppath\n";
  if ( $ostype eq 'win' ) {			# get OSDS for Windows from CATInstallPath
    if ( $ENV{CATInstallPath} =~ m/intel_a/ ) {
      $ENV{OSDS}='intel_a';
    }
    if ( $ENV{CATInstallPath} =~ m/win_b64/ ) {
      $ENV{OSDS}='win_b64';
    }
    print "       OSDS: $ENV{OSDS}\n";
  }

  # assign Official Program Name, like catstart
  for ($opt{run}) {
    /CNEXT.*/i && do { $official_program_name='CATIA V5'; last; };
    /DMU/i     && do { $official_program_name='DMU Navigator'; last; };
    /ENOVIA/i  && do { $official_program_name='ENOVIA'; last; };
    /Delmia/i  && do { $official_program_name='Delmia'; last; };
  }

  # remote or local running?
  if (defined($opt{remote})) {
  # remote running
    if (! defined($opt{user})) {
      die &printusage;
    }
    if (defined($opt{env}))    { $remotearg=$remotearg . " -env $opt{env}";	};
    if (defined($opt{direnv})) { $remotearg=$remotearg . " -d $opt{direnv}";	};
    if (defined($opt{object})) { $remotearg=$remotearg . " -object $opt{object}";	};
    if (defined($opt{run}))    { $remotearg=$remotearg . " -run $opt{run}";	};
    if (defined($opt{s}))      { $remotearg=$remotearg . " -s";			};
  }

  if ( defined($opt{catref}) ) {
    if ( $opt{catref} ne '?' ) {
      $opt{catref}=&trim_path($opt{catref});
      @catref=split /$seperator+/, $opt{catref};
      foreach (@catref) {
        if ( ! -d $_ ) {
          &print_error("Cannot find specified CATReferenceSettingPath \"$_\"");
          exit 1;
        }
      }
    }
  }
  if ( defined($opt{novar}) ) {
    $opt{nocaa}=1;
    $opt{noscripts}=1;
    $opt{nodlnames}=1;
    $opt{nohotfix}=1;
  }
}

if ( $app eq "nx" ) {
  $apppath=&getNXPath;
}

# check if installations are running from CABUMInstaller 2.x, files *.lck
$dummy=$apppath;
$dummy=$1 if $dummy =~ /^(.*).intel_a/;
$dummy=$1 if $dummy =~ /^(.*).win_b64/;
if ( -f $dummy . ".lck" ) {
  print("\n  !!! It seems that CABUMInstaller2 works on $dummy !!!\n");
  sleep 10;
  exit 1;
}

# define and create needed work directories
&get_mydirs;

# Check for optional additional customer specific extension of environment, works only for SCHWINDT
if ( -d catfile("c:","3dx","my","catia") ) {
  $environment_name_add = catfile($ENV{CAT_SITE_MY}, "env", $ENV{V5ENV});
  # suffix ".txt" should already be gone
  $environment_name_add =~ s/\.txt$//;
  $environment_name_add = $environment_name_add . ".add.txt";
  print ("    Test customer specific add: $environment_name_add\n");
  if ( -e $environment_name_add ) {
    print ("      Found!\n");
    &source_env($environment_name_add);		# source an optional environment add
  }
}

# source the projects (subenvironments)
if ( defined($opt{project}) ) {
  &print_h1 ("Source the projects ...");
  @v5project=split /\s+/, $opt{project};
  $dummy=1;
  foreach (@v5project) {
    #$_=$_ if $_;
    $_=$_ . ".txt"                              if -f $_ . ".txt";
    $_=catfile($ENV{V5PROJECTS}, $_)            if -f catfile($ENV{V5PROJECTS}, $_);
    $_=catfile($ENV{V5PROJECTS}, $_  . ".txt" ) if -f catfile($ENV{V5PROJECTS}, $_  . ".txt" );
    if ( -f "$_" ) {
      &print_h2("Project-file \"$_\"",$dummy);
	  $project_add=$_;
      &source_env($_);
    } else {
      &print_h2("Cannot find project-file \"$_\[.txt\]\"",$dummy);
    }
    $dummy=$dummy+1;
  }
}

# Check for optional additional customer specific extension of project, works only for SCHWINDT
if ( defined($opt{project}) && -d catfile("c:","3dx","my","catia") ) {
  $project_add =~ s/3dx(.)schwindt/3dx$1my/i;
  # suffix ".txt" should already be gone
  $project_add =~ s/\.txt$//;
  $project_add = $project_add . ".add.txt";
  print ("    Test customer specific add: $project_add\n");
  if ( -e $project_add ) {
    print ("      Found!\n");
    &source_env($project_add);		# source an optional project add
  }
}

# check and run customized scripts
&run_custom_scripts if ! defined $opt{noscripts};

if ( $app eq "catia" ) {
  # include CAA's
  &include_caa if ! defined $opt{nocaa};

  $ENV{CATReferenceSettingPath}=&trim_path($ENV{CATReferenceSettingPath});
  # Select CATReferenceSettingPath?
  if ( $opt{catref} eq '?' ) {
    $ENV{CATReferenceSettingPath}=&select_catref($ENV{CATReferenceSettingPath});
    $opt{catref}=$ENV{CATReferenceSettingPath};
  }
  $ENV{CATReferenceSettingPath}=$opt{catref} if ( defined($opt{catref}) );
  
  # extend CATReferenceSettingPath with an environment and user dependend directory,
  # used for DLNames for example
  $ENV{CATReferenceSettingPath}=$myautoref . $seperator . $ENV{CATReferenceSettingPath};

  # check for dlname import
  &import_dlnames if ! defined $opt{nodlnames};

  # generate the the customized titlebar
  &setTitlebar;

  # include HOTFIX
  if (! defined $opt{nohotfix}) {
    if ( defined($ENV{V5HOTFIX}) ) {
      &install_caa( 0, $ENV{V5HOTFIX}, "Insert HOTFIX \"$ENV{V5HOTFIX}\" ...");
    }
  }

  # delete UserSettings?
  if ( defined $opt{reset} && -d $ENV{CATUserSettingPath} ) {
    my $z1;
    &print_h1 ("Delete Usersettings ...");
    $z1=unlink glob( catfile($ENV{CATUserSettingPath}, ".*.CATSettings"));
    $z1=$z1+unlink glob( catfile($ENV{CATUserSettingPath},  "*.CATSettings"));
    &print_h2("$z1 files deleted!");
  }

  # license selected?
  #unlink catfile($myautoref, "Licensing.CATSettings")  if -f catfile($myautoref, "Licensing.CATSettings");
  #unlink catfile($myautoref, "ICMLicense.CATSettings") if -f catfile($myautoref, "ICMLicense.CATSettings");
  if ( defined $ENV{MYLIC} ) {
    if ($ENV{MYLIC} =~ m/.CATSettings/i) {
	  # old behavior => copy a file to autoref
      &print_h1 ("Get license file from $ENV{MYLIC} ...");
      if ( ! -f $ENV{MYLIC} || ! -r $ENV{MYLIC} ) {
        &print_warn("Cannot find license-file $ENV{MYLIC}, will skip this option!");
      }
	  else {
        &print_h2("Copy license file to $myautoref");
        copy($ENV{MYLIC},catfile($myautoref, "Licensing.CATSettings"));
        copy(catfile(dirname($ENV{MYLIC}), "ICMLicense.CATSettings"), catfile($myautoref, "ICMLicense.CATSettings")) if ( -f  catfile(dirname($ENV{MYLIC}), "ICMLicense.CATSettings") );
	  }
	}  
    else {
      # new behavior => generate and import a xml-File
      &print_h1("Write and import xml-File for $ENV{MYLIC} ...");
	  &import_license($ENV{MYLIC});
	}

    if ( -f catfile($ENV{CATUserSettingPath}, "_Licensing.CATSettings") ) {
      unlink catfile($ENV{CATUserSettingPath}, "Licensing.CATSettings") if -f catfile($ENV{CATUserSettingPath}, "Licensing.CATSettings");
    }
    else {
      move(catfile($ENV{CATUserSettingPath}, "Licensing.CATSettings"), catfile($ENV{CATUserSettingPath}, "_Licensing.CATSettings")) if -f catfile($ENV{CATUserSettingPath}, "Licensing.CATSettings");
    }
  }
  else {
    # no preselection of license
    move(catfile($ENV{CATUserSettingPath}, "_Licensing.CATSettings"), catfile($ENV{CATUserSettingPath}, "Licensing.CATSettings")) if -f catfile($ENV{CATUserSettingPath}, "_Licensing.CATSettings");
  }
}

# spacemouse-driver to start?
if ( defined $opt{spc} ) {
  &print_h1 ("Spacemousestart ...");
  &spc_start ($opt{spc}, $opt{spctty});
}

# REM a dump of the environment may be nice
&print_envdump(catfile($mywork, "set.txt"));

&print_h1 ("Starting $official_program_name program. Please wait ...");

if ( $app eq "catia" ) {
  # Start Remote Program
  if ( defined $opt{remote} ) {
    if ( $opt{remote} eq '.' ) {
      $ret=system "CATRemoteRunner $opt{remote} $opt{user} $remotearg &"
    }
    else {
      $ret=system "CATRemoteRunner $opt{remote} $opt{user} $remotearg"
    }
    if ( $ret != 0 ) {
      &print_error("$official_program_name Remote Program aborted");
    }
    exit $ret;
  }

  if ( $opt{run} eq 'ENOVIA' ) {
    # Start ENOVIA
    # ---> specific AIX
    if ( $plattform eq 'AIX'  ) {
      my($size, $dummy);
      $size=length $ENV{LIBPATH};
      $dummy=length $ENV{LD_LIBRARY_PATH};
      $size=$dummy if $dummy > $size;
      if ( $size > 950 ) {
        &print_warn("Your LIBPATH or LD_LIBRARY_PATH variable exceeds 950 characters ($size). Java may not run as expected!");
      }
    }
    # <--- specific AIX
    if (defined($opt{port}))    { $enovoptions="-port $opt{port}";	            };
    if (defined($opt{serv}))    { $enovoptions="$enovoptions -serv $opt{serv}";	    };
    if (defined($opt{senv}))    { $enovoptions="$enovoptions -senv $opt{senv}";	    };
    if (defined($opt{sdirenv})) { $enovoptions="$enovoptions -sdirenv $opt{sdirenv}"; };
    $ret=system "enovjavstart $enovoptions";
    if ( $ret != 0 ) {
      &print_error("ENOVIA Program aborted");
    }
  }
  else {
    # start a Program, not ENOVIA
    &set_os_dependend_variables;

    print "\n";
    &print_info("CATInstallPath=$ENV{CATInstallPath}");
    print "\n";
    &print_info("CATReferenceSettingPath=$ENV{CATReferenceSettingPath}");
    print "\n";
    &print_info("CATUserSettingPath=$ENV{CATUserSettingPath}");
    
    # start with low priority if variable CATLOWPRIORITY=1
    if ( $ENV{CATLOWPRIORITY} eq '1' && $ostype eq 'win' ) {
      &print_info("\n  CATLOWPRIORITY=1 detected");
      my($obj);
      Win32::Process::Open($obj, $$, 0);
      $obj->SetPriorityClass(Win32::Process::IDLE_PRIORITY_CLASS());
    }
	
    if ( defined $opt{run} ) {
	  # wla, 05.06.18: 3DExperience 2018x braucht dies, um Teile aus dem Browser nach V5 zu laden
	  # wla, 29.11.19: mit Variable No3DEXPRunOperation abschaltbar
	  if ( ! defined $ENV{No3DEXPRunOperation} ) {
	    # default
	    $opt{run}="$opt{run} -e 3DEXPRunOperation" if ( $opt{run} =~ /CNEXT.*/i || $opt{run} =~ /DMU.*/i );
      }
	  else {
	    # disable
	    $opt{run}="$opt{run}" if ( $opt{run} =~ /CNEXT.*/i || $opt{run} =~ /DMU.*/i );
	  }
      print "\n";	  
      if ( defined $opt{object} ) {
        &print_info("$opt{run} $opt{object}");
        $ret=system "$opt{run} $opt{object}";
      }
      else {
        &print_info("$opt{run}");
        $ret=system "$opt{run}";
      }
      if ( $ret != 0 ) {
        &print_error("$official_program_name aborted!");
      }
    }
  }
}

if ( $app eq "nx" ) {
  print "  UGII_BASE_DIR:         $ENV{UGII_BASE_DIR}\n";
  print "  UGII_ROOT_DIR:         $ENV{UGII_ROOT_DIR}\n";
  print "  UGS_LICENSE_SERVER:    $ENV{UGS_LICENSE_SERVER}\n" if ( defined $ENV{UGS_LICENSE_SERVER} );
  print "  SPLM_LICENSE_SERVER:   $ENV{SPLM_LICENSE_SERVER}\n";
  print "  UGS_LICENSE_BUNDLE:    $ENV{UGS_LICENSE_BUNDLE}\n";
  print "  HQM_LICSERVER:         $ENV{HQM_LICSERVER}\n" if ( defined $ENV{HQM_LICSERVER} );
  print "  HDDRUCK_LICENSE_FILE:  $ENV{HDDRUCK_LICENSE_FILE}\n" if ( defined $ENV{HDDRUCK_LICENSE_FILE} );
  print "  UGII_USER_PROFILE_DIR: $ENV{UGII_USER_PROFILE_DIR}\n";
  $ENV{UGII_UGDOC_BASE}="https://docs.plm.automation.siemens.com" if ( ! defined $ENV{UGII_UGDOC_BASE} );
  print "  UGII_UGDOC_BASE:       $ENV{UGII_UGDOC_BASE}\n";
  print "\n";

  # create directory $ENV{UGII_USER_PROFILE_DIR}, if variable is set
  if ( defined $ENV{UGII_USER_PROFILE_DIR} ) {
	if ( ! -d $ENV{UGII_USER_PROFILE_DIR} ) {
      eval { mkpath($ENV{UGII_USER_PROFILE_DIR}, 0, 0777) } || &print_warn("!!! Cannot create directory $ENV{UGII_USER_PROFILE_DIR} !!!") && sleep 5;
	}
  }
  
  # start with low priority if variable NXLOWPRIORITY=1
  if ( $ENV{NXLOWPRIORITY} eq '1' && $ostype eq 'win' ) {
    &print_info("\n NXLOWPRIORITY=1 detected\n");
    my($obj);
    Win32::Process::Open($obj, $$, 0);
    $obj->SetPriorityClass(Win32::Process::IDLE_PRIORITY_CLASS());
  }

  &print_info("$opt{run}");
  $ret=system "$opt{run}";
}

$ENV{CADRETURNCODE}=$ret;
print "\n";
&print_info("Returncode: $ret");

# check and run stop scripts
&run_stop_scripts if ( ! defined $opt{noscripts} || ! defined $opt{nocaa} );

sleep 3;
exit 3 if $ret!=0 ;

#####   -----   Functions   -----   #####
sub trim {
  my $string=shift;
  for ($string) {
    s/^\s+//;
    s/\s+$//;
  }
  return $string;  
}


sub trim_path {
  # remove leading and behind seperators, replace repeated seperators with only one
  $_=$_[0];
  s/^$seperator+//;
  s/$seperator+$//;
  s/$seperator+/$seperator/;
  $_;
}


sub getosds {
  if ( ! defined $ENV{OSDS} ) {
  my $KernelType;
    for ($plattform) {
      /AIX/   && do {
        $KernelType=&get_kerneltype;
    if ( $KernelType == 64 ) {
	  # we have a 64 bit kernel
          $ENV{OSDS}="aix_a64";
	  if ( system "$0 -env \"$opt{env}\" -direnv \"$opt{direnv}\" -novar -run \"which catstart\" 1>/dev/null 2>&1" ) {
            $ENV{OSDS}="aix_a";
	    if ( system "$0 -env \"$opt{env}\" -direnv \"$opt{direnv}\" -novar -run \"which catstart\" 1>/dev/null 2>&1" ) {
	      &print_h2("Found a 64 bit kernel, but neither 64 or 32 bit CATIA!");
	      exit 1;
	    }
	    else {
	      &print_h2("Your AIX kernel is 64 bit, but your CATIA seems to be 32 bit!");
	      &print_h2("May be, this is not what you want!");
	    }
	  }
	}
	else {
	  # we have a 32 bit kernel
	  $ENV{OSDS}="aix_a";
	  if ( system "$0 -env \"$opt{env}\" -direnv \"$opt{direnv}\" -novar -run \"which catstart\" 1>/dev/null 2>&1" ) {
	    $ENV{OSDS}="aix_a64";
 	    if ( system "$0 -env \"$opt{env}\" -direnv \"$opt{direnv}\" -novar -run \"which catstart\" 1>/dev/null 2>&1" ) {
	      &print_h2("Found a 32 bit kernel, but neither 32 or 64 bit CATIA!");
	      exit 1;
	    }
	    else {
	      &print_h2("Your AIX kernel is 32 bit, but your CATIA seems to be 64 bit!");
	      &print_h2("May be, this is not what you want!");
	    }
	  }
        }
        last; };
      /HP-UX/ && do {
        $ENV{OSDS}='hpux_b'; last; };
      /IRIX/  && do {
        $ENV{OSDS}='irix_a'; last; };
      /SunOS/ && do {
        $ENV{OSDS}='solaris_a'; last; };
      # For Windows, we set OSDS later by checking the CATInstallPath
      # OTHER
      &print_error("Plattform $^ is not supported");
      exit 1;
    }
  }
  else {
    &print_h2("OSDS already set to $ENV{OSDS}");
  }
  &print_h2($ENV{OSDS});
  if ( $ENV{OSDS} eq 'hpux_b' ) { 0 }
}


sub get_mydirs {
  # define and create the directories
  #			$mywork			# $home/.[catia|nx],   a working dir for user
  # 		$myenvwork;		# $mywork/[ENVNAME],   a working dir for user and env
  #			$myautoref;		# $myenvwork/autoref,  CATIA references for user and env
  #			$mytitlebar;	# $myenvwork/titlebar, CATIA titlebar
  $mywork=catfile($home, ".$app");
  $_=basename($environment_name);
  if ( $app eq "catia" ) {
    s/\.[^\.]*$//;			# eliminate extension
    $myenvwork=catfile($mywork, $_);
    $myautoref=catfile($myenvwork, "autoref");
    $mytitlebar=catfile($myenvwork, "titlebar");
  }
  # wla, 06.03.2020 v1.49: autoref vor jedem Start löschen
  rmtree $myautoref if -d $myautoref;
  foreach ($mywork, $myenvwork, $myautoref, $mytitlebar) {
    if (defined($_) && ! -d $_) {
      unless ( mkpath($_, 0, 0777) ) {
        &print_error("Cannot ceate directory $_!");
        exit 1;
      }
    }
  }
}


sub set_v5mem {
  # V5MEM, only if EXTSHM is set in $X11ini
  my $X11ini="/usr/lpp/X11/defaults/xserverrc";
  my $extshm_rdy;
  &print_info("Check the memory management on AIX \(V5MEM\)");
  if ( ! -f $X11ini || ! -r $X11ini ) {
    &print_warn("Cannot open $X11ini for check variable EXTSHM before set V5MEM!",2);
  }
  else {
    open X11ini, "<$X11ini";
    $extshm_rdy=-1;
    while (<X11ini>) {
      chomp;
      if ( /^(export E|E)XTSHM=\w/ ) {
        $extshm_rdy=2;
      }
      if ( /^export EXTSHM/ && $extshm_rdy == 2 ) {
        $extshm_rdy=1;
	      last;
      }
    }
    close X11ini;
    if ( $extshm_rdy != 1 ) {
      # EXTSHM=On not set in $X11ini
      if ( defined $ENV{V5MEM} ) {
        &print_warn("Cannot use V5MEM=$ENV{V5MEM}. Please set \"EXTSHM=On ; export EXTSHM\" in $X11ini!",2);
      }
      else {
        &print_warn("Please set \"EXTSHM=On ; export EXTSHM\" in $X11ini and set V5MEM=\[5-8\] for use >1GB RAM!",2);
      }
    }
    else {
      # EXTSHM=On set in $X11ini
      if ( defined $ENV{V5MEM} ) {
        if ( $ENV{V5MEM} =~ /[^5-8]/ ) {
          &print_warn("V5MEM=$ENV{V5MEM} is not allowed! please use V5MEM=\[5-8\]!",2);
          delete $ENV{V5MEM};
        }
        else {
          &print_h2("Found V5MEM=$ENV{V5MEM}",2);
        }
      }
      if ( ! defined $ENV{V5MEM} ) {
        &print_h2("Invalid or no V5MEM variable found, will calculate it automatically ...",2);
	$_=`/usr/sbin/lsattr -E -l sys0 -a realmem`;
	/^realmem\s(\d+)\s/;
	$_=$1;
	$ENV{V5MEM}=int($_ / 1024 / 256)+1;
	if ( $ENV{V5MEM} < 5 ) {
	  delete $ENV{V5MEM};
	  &print_h2("You have only $_ kBytes of physical memory, will not set V5MEM!",2);
	}
	else {
	  $ENV{V5MEM}=8 if $ENV{V5MEM} > 8;
	  &print_h2("You have $_  kBytes of physical Memory, will set V5MEM to $ENV{V5MEM}!",2);
	}
      }
    }
  }
}


sub select_catref {
  my $count=0;
  my @catref;
  @catref=split /$seperator+/, $_[0];
  &print_h1 ("Select Your CATReferenceSettingPath ...");
  if (  @catref ) {
    foreach (@catref) {
      $count=$count+1;
      print "   \($count\) ";
      if ( -d $_ ) {
#         if ( -r $_ ) { print "r" } else { print "-" }; # "-r" doesn't work on windows ACLs
#        if ( -w $_ ) { print "w" } else { print "-" }; # "-w" doesn't work on windows ACLs
        print " ";
        print "v";
      }
      else {
        print "?";
      }
      print " $_\n";
    }
    $count=$count+1;
    print "\n  $\($count\) All in listed order\n";
    $catref[$count-1]=$_[0];
    $count=$count+1;
    print "  $\($count\) Without\n";
    $catref[$count-1]=undef;
    print "  \n     v=Path available ?=Path not available\n\n\n";
    $_=-1;
    while ( /[^1-$count]/ ) {
      print "  Your selection: ";
      chomp ($_=<STDIN>);
    }
    print "\n  Selected CATRefererenceSettingPath: \"$catref[$_-1]\"\n";
    $catref[$_-1];
  }
  else {
    &print_error("The environment have no CATReferenceSettingPath set!");
    exit;
  }
}


sub eval_variables {
  my ($count, $var);
  # substitute the unix/win Variables with their value
  $_=$_[0];
  if ( /DSKEY_/ ) {				# DSKEY_-Variablen ersetzen
    s/DSKEY_USERID/$logname/g;
    s/DSKEY_TMPDIR/$temp/g;
  }

  s/"//g;	# eliminate all """

  if ($ostype eq 'win') {
    # Windows-Variables (%VAR%, CSIDL_*)
    s/%(CSIDL_\w+)%/$1/g;			# %CSIDL_VARIABLE% -> CSIDL_VARIABLE
    s/(CSIDL_\w+)/eval("Win32::GetFolderPath(Win32::$1)")/eg; # evaluate CSIDL_VARIABLE

    if ( /^(.*?)=.*/ ) {
      $var=$1;
      # let windows evaluate the variables
      $_=&trim(`"set $_ && set $var"`);
    }
  }

  # UNIX-Variables (maybe also WIN) ($VAR, ${VAR})
  $count=0;  
  if ($ostype eq 'unix') {
    # since 1.39 we do this only with unix
    while ( /\$/ ) {	  #
      # Attention with lines like CLASSPATH=\$CLASSPATH_JDBC_$OSDS:$CLASSPATH
      s/(?<=[^\\])\$(\w+)/\$ENV\{$1\}/gi;	# $var   but not \$var   -> $ENV{var}
	  s/(?<=[^\\])\$({[^}]+})/\$ENV$1/gi;	# ${var} but not \${var} -> $ENV{var}
      s/(^[^"].*$)/"$1"/;					# enclose all with ""
      $_=eval "$_";
      if ($ostype eq 'unix') { s/\\*\$/\$/g }	# \$     -> $

      $count=$count+1;
      if ( $count > 16 ) {
        print "\n";     
        print "!!!!! Problem when evaluating variables !!!!!\n";
        print "   Check $_\n";
        exit 1;
      }
	}
  }
  $_;
}


sub source_env {
  # source the environment
  my $ret="unknown";
  unless ( open (ENV, "< $_[0]") ) {
    &print_error("Cannot open environment-file \"$_[0]\"");
    exit 1;
  }
  else {
    my ($zeile, $var, $val);
    while (<ENV>) {
      chomp;
      if ( /TMSTMP/ ) {
        s/^.*TMSTMP\s*:\s*([0-9]+)/$1/g;
        $ENV{CATSTART_TMSTMP}=$_;			# like catstart
      }
      # keine Leerzeilen, Kommentarzeilen "=" muss enthalten sein
      if ( ! /^[#,!]/ && ! /^\s/ && /=/ ) {
        $_=&eval_variables($_);
        $var=$_; $val=$_;
        # $var durch \$var substituieren
	    if ( /^(.*)=.*/ ) { $var=$1 }
        if ( /^.*=(.*)/ ) { $val=$1 }
        $ENV{$var}=$val;
      }
      $ret="catia" if $var eq "CATGraphicPath";
      # 25.07.13, wla: CATInstallPath added, if a reduced environment without CATGraphicPath is sourced
      $ret="catia" if $var eq "CATInstallPath";
      $ret="nx" if $var eq "UGII_BASE_DIR";
    }
    close ENV;
  }
  return $ret;
}


sub include_caa {
# include caa-progs by starting customized scipts "start_*.pl" from caa-path
  &print_h1 ("Search for caa progs to include ...");
  if ( defined($ENV{V5CAA}) ) {
    my($i, @v5caapath, @v5caascripts, @v5caaversion, $v5caa2search, $v5caascript, $v5caaroot);
    &print_info("Test shell variable V5CAA, found");
    &print_h2("V5CAA=$ENV{V5CAA}");

    $ENV{V5CAA}=&trim_path($ENV{V5CAA});
    $ENV{V5CAA}=&eval_variables($ENV{V5CAA});
    @v5caapath=split /$seperator+/, $ENV{V5CAA};
    foreach (@v5caapath) {
	  $v5caa2search=$_;
      if (-d $v5caa2search) {
        &print_h2("Path $v5caa2search, search for caa ...");

        undef(@v5caascripts);
        # $i=0 each caa is declared (old method), $i=1 search in subdirectiories first level (preferred method), $i=2 never used
		for ($i=0; $i<=2; $i++) {
		  if ( ! @v5caascripts ) {
            find ( { preprocess => sub { my $depth = $File::Find::name =~ tr[/][]; return @_ if $depth <= $i; return; },
                         wanted => sub { my $depth = $File::Find::name =~ tr[/][]; push @v5caascripts,catfile($File::Find::name) if (-f $File::Find::name and /^start_.*\.pl$/); },
              },
              ($v5caa2search),
		    );
          }
        }
		
        foreach (@v5caascripts) {
          # Check if version is visible, from an info file version_XXX.txt
		  $v5caascript=$_;
          @v5caaversion=undef;
          @v5caaversion=glob (catfile (dirname($v5caascript), "version_*.txt"));
          if ( defined $v5caaversion[0] && -f @v5caaversion[0] ) {
            if ( $v5caaversion[0] =~ /version_(.*).txt/ ) {
              $v5caaversion[0]=$1;
            }
          }
          $v5caaroot=dirname($v5caascript);
          if ( defined $v5caaversion[0] ) {
            &print_h2("caa $v5caaroot found, Version " . $v5caaversion[0])
          }
          else {
            &print_h2("caa $v5caaroot found")
          }
          if ( &isExecutable($v5caascript) ) {
            &print_h2("Execute $v5caascript",3);
            unless ( do $v5caascript )  { 
              &print_error("Error in script $v5caascript\n       $@\n",4) if $@;
            }
          }
          else { &print_h2("Not executable file $v5caascript"); }
        }
        if ( ! @v5caascripts ) {
          &print_h2("No start-script found in $v5caa2search, will skip this path!",2);
        }
      }
      else { &print_h2("Path $v5caa2search not found!"); }
    }
  }
  else { &print_info("Test shell variable V5CAA, not set"); }
}


sub install_caa {
# install caa programs by insert/append the caa-path in the environment
# to call with &install_caa(0|1, [CAAPATH], [optional message])
#                           0-insert, 1-append
  my ($v5caapath, @v5path, $key);
  $v5caapath=$_[1];
  if ( ! ($v5caapath =~ /$ENV{OSDS}$/) ) {
    # caa path maybe declared with or without $OSDS
    $key=catfile($v5caapath, $ENV{OSDS});
    $v5caapath=$key if -d $key;
  }
  my %v5caaenv= (
    "CATEnvironment"		=>	$v5caapath,
    "CATInstallPath"		=>	$v5caapath,
    "PATH"			=>	catfile($v5caapath,"code","bin") . $seperator . catfile($v5caapath,"code","command"),
    "LIBPATH"			=>	catfile($v5caapath,"code","bin"),
    "LD_LIBRARY_PATH"		=>	catfile($v5caapath,"code","bin"),
    "LD_LIBRARYN32_PATH"	=>	catfile($v5caapath,"code","bin"),
    "SHLIB_PATH"		=>	catfile($v5caapath,"code","bin"),
    "CATDLLPath"		=>	catfile($v5caapath,"code","bin"),
    "CATICPath"			=>	catfile($v5caapath,"code","productIC"),
    "CATCommandPath"		=>	catfile($v5caapath,"code","command"),
    "CATDictionaryPath"		=>      catfile($v5caapath,"code","dictionary"),
    "CATDocView"		=>	catfile($v5caapath,"doc"),
    "CATReffilesPath"		=>	catfile($v5caapath,"reffiles"),
    "CATFontPath"		=>	catfile($v5caapath,"resources","fonts"),
    "CATGalaxyPath"		=>	catfile($v5caapath,"resources","galaxy"),
    "CATGraphicPath"		=>	catfile($v5caapath,"resources","graphic") . $seperator . catfile($v5caapath,"resources","graphic","icons") . $seperator . catfile($v5caapath,"resources","graphic","figures") . $seperator . catfile($v5caapath,"resources","graphic","splashscreens") . $seperator . catfile($v5caapath,"resources","graphic","symbols") . $seperator . catfile($v5caapath,"resources","graphic","textures"),
    "CATMsgCatalogPath"		=>	catfile($v5caapath,"resources","msgcatalog"),
    "CATFeatureCatalogPath"	=>	catfile($v5caapath,"resources","featurecatalog"),
    "CATDefaultCollectionStandard"=>	catfile($v5caapath,"resources","standard"),
    "CATKnowledgePath"		=>	catfile($v5caapath,"resources","knowledge"),
    "CATStartupPath"		=>	catfile($v5caapath,"startup"),
    "CATW3ResourcesPath"	=>	catfile($v5caapath,"docs"),
  );

  if ( defined $_[2] ) {
    &print_h1 ($_[2]);
  }
  elsif ( $_[0] eq 0 ) {
    &print_h1 ("Insert caa-path $v5caapath ...");
  }
  else {
    &print_h1 ("Append caa-path $v5caapath ...");
  }

  if ( ! -d $v5caapath ) {
    &print_h2("Path \"$v5caapath\" not found, will skip this option!",2);
  }
  else {
    foreach $key ( keys(%v5caaenv) ) {
      @v5path=split /$seperator/, $v5caaenv{$key};
      foreach (@v5path) {
        if ( -d $_ ) {
          if ( $_[0] eq 0 ) {	# insert
	        $ENV{$key}=$_ . $seperator . $ENV{$key};
          }
	      else {			# append
	        if ( defined $ENV{$key} ) {
	          $ENV{$key}=$ENV{$key} . $seperator . $_;
	        }
	        else {
	          $ENV{$key}=$_;
	        }
	      }
        }
      }
    }
  }
}


sub run_custom_scripts {
# run perl scripts "start_*.pl" from paths declared in shell variable
# SCHWINDTSCRIPTS and V5SCRIPTS
  my(@sctscriptpath, @schwindtscripts);
  &print_h1 ("Search for customized perl-scripts ...");
  if ( defined($ENV{SCHWINDTSCRIPTS}) ) {
    &print_info("Test shell variable SCHWINDTSCRIPTS, found");
    &print_h2("SCHWINDTSCRIPTS=$ENV{SCHWINDTSCRIPTS}");
  }
  if ( defined($ENV{V5SCRIPTS}) ) {
    &print_info("Test shell variable V5SCRIPTS, found");
    &print_h2("V5SCRIPTS=$ENV{V5SCRIPTS}");
  }
  if ( defined ($ENV{SCHWINDTSCRIPTS}) || defined ($ENV{V5SCRIPTS}) ) {
    $ENV{SCHWINDTSCRIPTS}=$ENV{SCHWINDTSCRIPTS} . $seperator . $ENV{V5SCRIPTS} 
  }
  if ( defined($ENV{SCHWINDTSCRIPTS}) ) {
    $ENV{SCHWINDTSCRIPTS}=&trim_path($ENV{SCHWINDTSCRIPTS});
    $ENV{SCHWINDTSCRIPTS}=&eval_variables($ENV{SCHWINDTSCRIPTS});
    
    @sctscriptpath=split /$seperator+/, $ENV{SCHWINDTSCRIPTS};
    foreach (@sctscriptpath) {
      if (-d $_) {
        &print_h2("Search for scripts in $_");
        @schwindtscripts=sort( glob (catfile($_, "start_*.pl") ) );
        foreach (@schwindtscripts) {
          if ( &isExecutable($_) ) {
            &print_h2("Execute $_",3);
	    unless ( do $_ )  { 
              &print_error("Error in script $_\n       $@\n",3) if $@;
            }
          }
	  else { &print_h2("Not executable file $_"); }
        }
      }
      else { &print_h2("Path $_ not found!"); }
    }
  }
  else { &print_info("Test shell variable SCHWINDTSCRIPTS, not set"); }
}


sub run_stop_scripts {
# run perl scripts "stop_*.pl" from paths declared in shell variable
# V5CAA, SCHWINDTSCRIPTS and V5SCRIPTS
  if ( defined($ENV{V5CAA}) || defined($ENV{SCHWINDTSCRIPTS}) ) {
    &print_h1 ("Search for scripts stop_\*.pl in V5CAA and V5SCRIPTS ...") if defined $ENV{V5CAA};
    &print_h1 ("Search for scripts stop_\*.pl in SCHWINDTSCRIPTS ...") if ! defined $ENV{V5CAA};
  }
  if ( defined($ENV{V5CAA}) ) {
    my($i, @v5caapath, @v5caascripts, @v5caaversion, $v5caa2search, $v5caascript);
    $ENV{V5CAA}=&trim_path($ENV{V5CAA});
    $ENV{V5CAA}=&eval_variables($ENV{V5CAA});
    @v5caapath=split /$seperator+/, $ENV{V5CAA};
    foreach (@v5caapath) {
	  $v5caa2search=$_;
      if (-d $v5caa2search) {
        &print_h2("Path $v5caa2search, search for scripts ...");
        undef(@v5caascripts);
        # $i=0 each caa is declared (old method), $i=1 search in subdirectiories first level (preferred method), $i=2 never used
		for ($i=0; $i<=2; $i++) {
		  if ( ! @v5caascripts ) {
            find ( { preprocess => sub { my $depth = $File::Find::name =~ tr[/][]; return @_ if $depth <= $i; return; },
                         wanted => sub { my $depth = $File::Find::name =~ tr[/][]; push @v5caascripts,catfile($File::Find::name) if (-f $File::Find::name and /^stop_.*\.pl$/); },
              },
              ($v5caa2search),
		    );
          }
        }
        foreach (@v5caascripts) {
		  $v5caascript=$_;
          if ( &isExecutable($v5caascript) ) {
            &print_h2("Execute $v5caascript",3);
            unless ( do $v5caascript )  { 
              &print_error("Error in script $v5caascript\n       $@\n",4) if $@;
            }
          }
          else { &print_h2("Not executable file $v5caascript"); }
        }
      }
    }
  }
    
  if ( defined($ENV{SCHWINDTSCRIPTS}) ) {
    my(@sctscriptpath, @schwindtscripts, $sctscript2search, $schwindtscript);
#    next 2 lines already done    
#    $ENV{SCHWINDTSCRIPTS}=&trim_path($ENV{SCHWINDTSCRIPTS});
#    $ENV{SCHWINDTSCRIPTS}=&eval_variables($ENV{SCHWINDTSCRIPTS});
    @sctscriptpath=split /$seperator+/, $ENV{SCHWINDTSCRIPTS};
    foreach (@sctscriptpath) {
	  $sctscript2search=$_;
      &print_h2("Path $sctscript2search, search for scripts ...");
      if (-d $sctscript2search) {
        @schwindtscripts=sort( glob (catfile($sctscript2search, "stop_*.pl") ) );
        foreach (@schwindtscripts) {
		  $schwindtscript=$_;
          if ( &isExecutable($schwindtscript) ) {
            &print_h2("Execute $schwindtscript",3);
            unless ( do $schwindtscript )  { 
              &print_error("Error in script $schwindtscript\n       $@\n",3) if $@;
            }
          }
          else { &print_h2("Not executable file $schwindtscript"); }
        }
      }
    }
  }
}


sub import_dlnames {
# import dlnames from files declared in the shell variable V5DLNAMES
# also the user specific file mydlnames.txt and mydlnames_[ENV] are supported
# the format of the files must be like the original format from DS, 
# CATIA.dcls files (V4 declarations) are also supported!
# the import is made to directory $myautoref
  my(@v5dlnames); my ($dldir, $mydlnames, $mydlnames2, $dlnames_log, $catref_original, $catref_mydlnames, $tp1, $tp2); my (%dlnamehash);
  &print_h1 ("Search for dlnames to import ...");
  if ( defined($ENV{V5DLNAMES}) ) {
    &print_info("Test shell variable V5DLNAMES, found");
    &print_h2("V5DLNAMES=$ENV{V5DLNAMES}");
    
    # Support for specific dlnames per user and per env
    $dldir=$mywork;
    if ( ! -d $dldir ) {
      unless ( mkpath($dldir, 0, 0777) ) {
        &print_error("Cannot ceate directory $dldir for my dlnames directory!");
        exit 1;
      }
    }
    
    $mydlnames=catfile($dldir, "mydlnames.txt" );
    if ( -f $mydlnames && -r $mydlnames ) {
      &print_info("Found user specific dlnames $mydlnames");
      $ENV{V5DLNAMES}=$ENV{V5DLNAMES} . $seperator . $mydlnames;
    }
    $mydlnames=catfile($dldir, "mydlnames_" . basename $ENV{CATEnvName} );
    if ( -f $mydlnames && -r $mydlnames ) {
      &print_info("Found user and environment specific dlnames $mydlnames");
      $ENV{V5DLNAMES}=$ENV{V5DLNAMES} . $seperator . $mydlnames;
    }
  
    $ENV{V5DLNAMES}=&trim_path($ENV{V5DLNAMES});    
    $ENV{V5DLNAMES}=&eval_variables($ENV{V5DLNAMES});
    @v5dlnames=split /$seperator+/, $ENV{V5DLNAMES};
    # open temporary file for concat all DL-Names
    $mydlnames=catfile($myenvwork, "dlnames_temp.txt" );
    $mydlnames2=catfile($myenvwork, "dlnames_temp2.txt" );
    unless ( open (V5DLNAME_TEMP, "+> $mydlnames") ) {
      &print_error("Cannot open temporary file $mydlnames");
      exit 1;
    }
    else {
      foreach (@v5dlnames) {
        unless ( open (V5DLNAME, "< $_") ) {
          &print_warn("Cannot open  $_ for dlname import!",2);
        }
        else {
	      &print_h2("Import dlnames from $_");
          while (<V5DLNAME>) {
            chomp;
            if ( ! /^[#,;,!]/ && ! /^\s$/ && /^[^;!].*;.*;.*;*$/ ) {
	          # file in original V5 format
	          print V5DLNAME_TEMP "$_\n";
  	        }
	        if ( /^catia.(model|session)\s*=\s*'.*\".*\"\s*,\s*".*\"\s*'\s*;\s*$/i ) {
	          # file in V4 format
	          /,\s*"([^"]*)"/;		# logical
	          $tp1=$1;
	          /=\s*'\s*"([^"]*)"/;	# physical
 	          $tp2=&eval_variables($1);
 	          print V5DLNAME_TEMP "$tp1;$tp2;$tp2;\n";
	        }
          }
          close V5DLNAME;
        }
      }
	  
  	  # --> wla, 07.05.2020: priority of DLNAMES, last wins
	  #   without this block, first wins
      seek(V5DLNAME_TEMP, 0, 0);
	  while (<V5DLNAME_TEMP>) {
        chomp;
		($tp1, $tp2) = split(/;/, $_, 2);
		$dlnamehash{$tp1} = $tp2;
	  }
      unless ( open (V5DLNAME_TEMP2, "> $mydlnames2") ) {
        &print_error("Cannot open temporary file $mydlnames2");
        exit 1;
      }
      seek(V5DLNAME_TEMP, 0, 0);
 	  while (<V5DLNAME_TEMP>) {
        chomp;
		($tp1, $tp2) = split(/;/, $_, 2);
		print V5DLNAME_TEMP2 "$tp1;$dlnamehash{$tp1}\n";
	  }
      close V5DLNAME_TEMP2;
	  $mydlnames=$mydlnames2;
	  # <--
	  
      close V5DLNAME_TEMP;
    }
	
    $catref_mydlnames=$myautoref;
    # delete the DLNames.CATSettings before import  - makes trouble if the file
    # exists and no entries in v5dlnames.txt
    if ( -e catfile($catref_mydlnames, "DLNames.CATSettings" ) ) {
      unlink catfile($catref_mydlnames, "DLNames.CATSettings");
    }
    if ( -s $mydlnames ) {
      # preserve original CATReferenceSettingPath
      $catref_original=$ENV{CATReferenceSettingPath};
      # set new CATReferenceSettingPath in which store dlnames
      $ENV{CATReferenceSettingPath}=$catref_mydlnames;
      $dlnames_log=catfile($myenvwork, "mydlnames.log");
      system "CATSysDLExport -admin -nocheck -il \"$mydlnames\" -r \"$dlnames_log\"";
      $ENV{CATReferenceSettingPath}=$catref_original;
    }
    else {
      &print_warn("Nothing to import, check the dlnames!");
    } 
  }
  else { &print_info("Test shell variable V5DLNAMES, not set!"); }
}

sub import_license {
  my ($licdeb, $licvalue, $licsize, $licxml, $catref_original, $pcfg, $mypcfg); my $lic_is_slt='no';

  # start writing the xml file
  $licxml=catfile($myenvwork, "mylic.xml" );
  unless ( open (LIC_TEMP, "> $licxml") ) {
    &print_error("Cannot open temporary file $licxml");
    exit 1;
  } else {
    print LIC_TEMP '<?xml version="1.0" encoding="UTF-8"?>' . "\n";
	print LIC_TEMP '<!DOCTYPE CATSettingRepository [' . "\n";
	print LIC_TEMP '<!ELEMENT CATSettingRepository	(Attribute*)>' . "\n";
	print LIC_TEMP '<!ATTLIST CATSettingRepository	Name NMTOKEN #REQUIRED>' . "\n";
	print LIC_TEMP '<!ELEMENT Attribute	(Value*)>' . "\n";
	print LIC_TEMP '<!ATTLIST Attribute	Name CDATA #REQUIRED' . "\n";
	print LIC_TEMP '                   	Type CDATA #REQUIRED' . "\n";
	print LIC_TEMP '                   	Size NMTOKEN #REQUIRED' . "\n";
	print LIC_TEMP '                   	Lock (Locked|JustLocked|Unlocked) "JustLocked">' . "\n";
	print LIC_TEMP '<!ELEMENT Value	(#PCDATA)>' . "\n";
	print LIC_TEMP ']>' . "\n";
	print LIC_TEMP "\n";
	print LIC_TEMP '<CATSettingRepository Name="licensing">' . "\n";
  }

  $pcfg=catfile($apppath, "control", "PCFG.list");
  # own file, special for configurations with addons
  $mypcfg=catfile($ENV{CAT_SITE_ALL}, "licenses", "catia", "myPCFG.list");
  foreach ( "$pcfg", "$mypcfg" ) {
	$pcfg=$_;
    if ( -f $pcfg  ) {
	  open PCFG, "<$pcfg";
      while (<PCFG>) {
        chomp;
	    if ( /\.prd|\.slt/ ) {
	      if ( /\.prd/ ) { $licvalue=$_ };
		  if ( /\.slt/ ) { $licvalue="_" . $_ };
		  # we need a "+" only if no addons are defined
		  if ( /\.slt$/ ) { $licvalue="_" . $_ . "+" };
		  $licdeb="LICDEB_" . $licvalue;
		  $licsize=length($licvalue);
		  print LIC_TEMP '<Attribute Name="' . $licdeb . '" Type="char" Size="' . $licsize . '">' . "\n";
		  print LIC_TEMP '<Value>' . "Aucune" . '</Value>' . "\n";
		  print LIC_TEMP '</Attribute>' . "\n";
		}
	  }
	  close PCFG;
	}
	else {
      &print_h2("No optional file $pcfg found!");	
	}
  }
	
  foreach ( (split / /, $_[0]) ) {
    $licvalue=$_;
    /(\w*)-*(\w*)/;
    # analyze the license string and form it proper
    if ( -f catfile($apppath, "control", $licvalue . ".prd") ) {
      $licvalue=$licvalue . ".prd";
	  $licdeb="LICDEB_" . $licvalue;
    }
    else {
	  $lic_is_slt="yes";
      $licvalue="_" . $1 . ".slt+" . $2;
	  $licdeb="LICDEB_" . $licvalue;
    }
    $licsize=length($licvalue);
    # complete the xml file
	print LIC_TEMP '<Attribute Name="' . $licdeb . '" Type="char" Size="' . $licsize . '">' . "\n";
	print LIC_TEMP '<Value>' . $licvalue . '</Value>' . "\n";
	print LIC_TEMP '</Attribute>' . "\n";
  }

  print LIC_TEMP '<Attribute Name="La_Liste_des_licences.nb" Type="int" Size="8" Lock="JustLocked">' . "\n";
  print LIC_TEMP '</Attribute>' . "\n";
  
  # now we have the accurate xml file
  print LIC_TEMP '</CATSettingRepository>' . "\n";
  close LIC_TEMP;
  
  if ( $lic_is_slt eq "no" ) { 
    &print_warn("You cannot start only with product license(s)!");
  }
  else {
    &print_h2("OK");
  }
	  
  # import the xml file
  # preserve original CATReferenceSettingPath
  $catref_original=$ENV{CATReferenceSettingPath};
  # set new CATReferenceSettingPath in which store Licensing.CATSettings
  $ENV{CATReferenceSettingPath}=$myautoref;
  system "CATBatImpXMLSet.exe \"$licxml\" -admin 1>NUL 2>&1";
  $ENV{CATReferenceSettingPath}=$catref_original;  
}


sub set_os_dependend_variables {
  if ( $os_var_like_ds eq 'yes' ) {          # OS variables like Dassault Systemes
    if ( $plattform eq 'AIX' ) {
      # ---> specific AIX
      $ENV{MALLOCTYPE}="buckets";
      if ( $ENV{OSDS} eq 'aix_a64' ) {
        # 64 bit CATIA
        $ENV{MALLOCBUCKETS}="number_of_buckets:128,bucket_sizing_factor:16,blocks_per_bucket:512";
        # $ENV{LDR_CNTRL}="PREREAD_SHLIB\@MAXDATA=0x0200000000000000"; # V5R15
        $ENV{LDR_CNTRL}="PREREAD_SHLIB";                               # V5R16
      }
      else {
        # 32 bit CATIA
        $ENV{MALLOCBUCKETS}="number_of_buckets:64,bucket_sizing_factor:8,blocks_per_bucket:512";
        &set_v5mem;  # set V5MEM, depends of EXTSHM, V5MEM, physical memory
        if ( $ENV{V5MEM} =~ /^[5-8]$/ ) {
          $ENV{LDR_CNTRL}="PREREAD_SHLIB\@MAXDATA=0x$ENV{V5MEM}0000000";
          $ENV{EXTSHM}="ON";
          $ENV{MW_GMA_VADDR}="0xE8000000";
          $ENV{JIMSHMATNUM}=0 if $ENV{LANG} =~ /Ja_JP/;
        }
        else {
          $ENV{LDR_CNTRL}="PREREAD_SHLIB";
        }
      }
      # <--- specific AIX
    }
    elsif ( $plattform eq 'HP-UX' ) {
      # ---> specific HP-UX
      $ENV{_HP_DLDOPTS}='-symtab_enable -symtab_size 70000';
      # <--- specific HP-UX
    }
  }
  
  else {                            # our own OS variables, partially from DC
    if ( $plattform eq 'AIX' ) {
      # ---> specific AIX
      $ENV{LANG}="en_US" if ! defined $ENV{LANG};  
      $ENV{MALLOCTYPE}="buckets";
      if ( $ENV{OSDS} eq 'aix_a64' ) {
        # 64 bit CATIA
        $ENV{MALLOCBUCKETS}="number_of_buckets:128,bucket_sizing_factor:16,blocks_per_bucket:512";
        $ENV{LDR_CNTRL}="PREREAD_SHLIB\@MAXDATA=0x0200000000000000";
      }
      else {
        # 32 bit CATIA
        my $KernelType=&get_kerneltype;
        $ENV{MALLOCBUCKETS}="number_of_buckets:64,bucket_sizing_factor:8,blocks_per_bucket:512";
        $ENV{LDR_CNTRL}="PREREAD_SHLIB";
        &set_v5mem;  # set V5MEM, depends of EXTSHM, V5MEM, physical memory
        if ( $ENV{V5MEM} =~ /^[5-8]$/ ) {
          if ( $KernelType == 64 ) { 
            $ENV{LDR_CNTRL}="PREREAD_SHLIB\@MAXDATA=0xD0000000\@DSA";
            $ENV{MW_GMA_VADDR}="0xD8000000";
          }
          else {
            $ENV{LDR_CNTRL}="PREREAD_SHLIB\@MAXDATA=0x$ENV{V5MEM}0000000";
            $ENV{MW_GMA_VADDR}="0xE8000000";
          }
          $ENV{EXTSHM}="ON";
          $ENV{JIMSHMATNUM}=0 if $ENV{LANG} =~ /[jJ]a_JP.*/;
        }
      }
      # <--- specific AIX
    }
    elsif ( $plattform eq 'IRIX' ) {
      # ---> specific IRIX
      $ENV{LANG}="en" if ! defined $ENV{LANG};  
      $ENV{MW_GMA_VADDR}="0x78000000";
      # use 4MB Pagesize for DSOs
      if (`systune -f coalesced_precoalesce | cut -d= -f2 | cut -d'(' -f1` == 1) {
        $ENV{MW_GMA_VADDR}="0x79000000";
        my $PAGE_SIZE = "4M";
        my $MMAP_SIZE = "4M";
        my $ALLOC_PREF = "0x400000-0x0f000000,u:0x20000000-0x78000000,d";
        $ENV{_RLD_ARGS_BIG_PAGES}="-big_page_load -big_page_pagesize $PAGE_SIZE -big_page_mmap_size $MMAP_SIZE -big_page_allocation_preference $ALLOC_PREF";
      }
      # to see the Icons on SGI again, performance is reduced with large models
      # but it's more colored
      $ENV{SGI_NO_OVL}="YES";
      # <--- specific IRIX
    }
    elsif ( $plattform eq 'HP-UX' ) {
      # ---> specific HP-UX
      $ENV{LANG}="en_US.iso88591" if ! defined $ENV{LANG};
      $ENV{_HP_DLDOPTS}="-symtab_enable -symtab_size 70000";
      $ENV{M_MXFAST}="512";
      $ENV{M_NLBLKS}="100";
      $ENV{M_VERBOSE}="1";
      $ENV{M_PREALLOC}="200000000";
      $ENV{HP_PHIGS_FAS_BUNCH}="1";
      $ENV{HP_PHIGS_BUNCH_DELAY}="4";
      $ENV{SB_DL_REDUNDANCY_SEARCH_LIMIT}="0";
      # <--- specific HP-UX
    }
    elsif ( $plattform eq 'SunOS' ) {
      # ---> specific SunOS
      # <--- specific SunOS
    }
  }
}


sub setTitlebar {
  # Set useful informations in the titlebar
  # we extend the CATMsgCatalogPath by insert our own path and copy
  # an original CATIA.CATNls file in this path with modifying
  # the string ApplicationFrame.Title

  my ($catnlsorg, $catnls);
  my %tb = ( 'conf_dir' => undef,
             'conf'		=> undef,
			 'my'		=> undef,
			 'default'	=> undef,
			 'tb.env'	=> undef,
			 'tb.rel'	=> undef,
			 'tb.spk'	=> "0",
			 'tb.hfx'	=> "0",
			 'tb.os'	=> $ENV{OSDS},
			 'tb.lic'	=> $opt{lic},
			 'tb.ip'	=> $apppath,
			 'tb.user'	=> $logname,
			 'tb.host'	=> $hostname,
			 'tb.date'	=> $date_s,
			 'tb.time'	=> $time_s,
			 'tb.prj'	=> $ENV{MYPROJECT}, );
			 
  $tb{'default'}="tb.env   tb.reltb.spktb.hfx,tb.os   tb.lic";
  $tb{'my'}=$tb{'default'};

  $tb{'conf_dir'}='c:\catia\sct\fdec\site\all' if $ostype eq 'win';
  $tb{'conf_dir'}='C:\3dx\my\catia\all'        if ( -d 'C:\3dx\my\catia\all' && $ostype eq 'win' );
  $tb{'conf_dir'}='/catcus5/fdec/site/all'     if $ostype eq 'unix';
  $tb{'conf'}=catfile($tb{'conf_dir'}, "titlebar.conf");
  
  if ( ! -d $tb{'conf_dir'} ) {
    # this is not our installation
  }
  
  else {
    # we found our installation
    &print_h1("Generate the titlebar ...");
    &print_info("To customize, edit the file $tb{conf}");

    if ( ! -f $tb{"conf"} ) {
      unless ( open (TB_CONF, "> $tb{'conf'}") ) {
        &print_h2("Cannot write $tb{'conf'}");
      }
      else {
        print TB_CONF "# titlebar.conf     by walter.laub\@schwindt.eu\n";
        print TB_CONF "#                   c(2020) Schwindt CAD/CAM-Technologie GmbH\n\n";
        print TB_CONF "# Please customize here the CATIA titlebar.\n";
        print TB_CONF "# This file is evaluated by sctcatstart.pl.\n\n";
        print TB_CONF "# You may use the variables:\n";
        print TB_CONF "#   tb.env       environment name\n";
        print TB_CONF "#   tb.rel       catia release\n";
        print TB_CONF "#   tb.spk       catia servicepack\n";
        print TB_CONF "#   tb.hfx       catia hotfix\n";
        print TB_CONF "#   tb.os        catia plattform\n";
        print TB_CONF "#   tb.lic       selected license in CABUM, if\n";
        print TB_CONF "#   tb.ip        install path\n";
        print TB_CONF "#   tb.user      user\n";
        print TB_CONF "#   tb.host      hostname\n";
        print TB_CONF "#   tb.date      date\n";
        print TB_CONF "#   tb.time      time\n\n";
        print TB_CONF "#   tb.prj       selected project in CABUM, if\n";
        print TB_CONF "# Syntax:\n";
        print TB_CONF "#   tb=\"Your new titlebar\"\n\n";
        print TB_CONF "# Examples:\n";
        print TB_CONF "#   tb=\"none\"     - deactivate the customized titlebar\n";
        print TB_CONF "#   tb=\"tb.env   tb.reltb.spktb.hfx,tb.os\@tb.ip   tb.user\@tb.host   tb.date,tb.time\"\n";
        print TB_CONF "#   tb=\"tb.env   tb.reltb.spktb.hfx,tb.os   tb.user\@tb.host\"\n\n";
        print TB_CONF "tb=\"$tb{'default'}\"\n";
        close TB_CONF;
      }
    }
 
    else {
      unless ( open (TB_CONF, "< $tb{'conf'}") ) {
        &print_h2("Cannot open $tb{'conf'}");
      }
      else {
        while (<TB_CONF>) {
          chomp;
          if ( /^[t,T][b,B]=.*$/ ) {
            s/^tb=\"//; # eliminate tb="
            s/^tb=//;   # eliminate tb=
            s/\".*$//;     # elimnate \" at end of string
            $tb{'my'}=$_;
            last;
          }
        }
        close TB_CONF;
      }    
    }
  }
  if ( $tb{'my'} ne "none" )  {
    # we copy the original file from the CATInstallPath
    # we search for the original CATIA.CATNls file
    $catnls=catfile($apppath, "resources", "msgcatalog", "CATIA.CATNls");
    $catnlsorg=$catnls if ( -f $catnls )
  }

  if ( defined($catnlsorg) && -f $catnlsorg 
	&& defined($apppath) && -d $apppath) {
    
	$tb{'tb.env'} = $ENV{V5ENV};
	
	foreach ( glob(catfile($apppath, "*.lp")) ) {
      unless ( open (PARAMFILE, "< $_") ) {
        &print_error("Cannot open $_");
        exit 1;
      }
	  while (<PARAMFILE>) {
	    chomp;
#	    if ( /.*(V\d*R\d*).*/ ) {
	    if ( /.*(V\d.*?)[\._].*/ ) {
	      $tb{'tb.rel'}=$1;
		  last;
		}
	  }
      close PARAMFILE;
	}
	
	$_=catfile($apppath, "LevelSPK.txt");
	if ( -f $_ ) {
      unless ( open (PARAMFILE, "< $_") ) {
        &print_error("Cannot open $_");
        exit 1;
      }
	  while (<PARAMFILE>) {
	    chomp;
	    if ( /.*Service Pack (\d*).*/ ) {
	      $tb{'tb.spk'}=$1;
		  last;
		}
	  }
      close PARAMFILE;
	}

	$_=catfile($apppath, "LevelHFX.txt");
	if ( -f $_ ) {
      unless ( open (PARAMFILE, "< $_") ) {
        &print_error("Cannot open $_");
        exit 1;
      }
	  while (<PARAMFILE>) {
	    chomp;
	    if ( /.*[H,h]ot *[F,f]ix (\d*).*/ ) {
	      $tb{'tb.hfx'}=$1;
		  last;
		}
	  }
      close PARAMFILE;
	}

    # V6
	$_=catfile($apppath, "..", "SftMgt", "Install.xml");
	if ( -f $_ ) {
      unless ( open (PARAMFILE, "< $_") ) {
        &print_error("Cannot open $_");
        exit 1;
      }
	  while (<PARAMFILE>) {
	    chomp;
	    if ( /hotFixPackagingVersionShortName=".*?HF(\d*)".*/ ) {
	      $tb{'tb.hfx'}=$1;
		  last;
		}
	  }
      close PARAMFILE;
	}
    
	# this is "old behavior" with Licensefile attached, name of license in path
	$_=$tb{'tb.lic'};
    if ( /.*\\(.*).*\\/ ) {
      $tb{'tb.lic'}=$1;
	}
	
	$_=$tb{'my'};
	s/tb.env/$tb{'tb.env'}/g;
    s/tb.rel/$tb{'tb.rel'}/g;
    s/tb.spk/SP$tb{'tb.spk'}/g;
    s/tb.hfx/HF$tb{'tb.hfx'}/g;
    s/tb.os/$tb{'tb.os'}/g;
	s/tb.lic/$tb{'tb.lic'}/g;
    s/tb.ip/$tb{'tb.ip'}/g;
    s/tb.user/$tb{'tb.user'}/g;
    s/tb.host/$tb{'tb.host'}/g;
    s/tb.date/$tb{'tb.date'}/g;
    s/tb.time/$tb{'tb.time'}/g;
    s/tb.prj/$tb{'tb.prj'}/g;
	s/\s+$//;
    $tb{'my'}=$_;
	
	# now we copy and modify our CATIA.CATNls
    $catnls=catfile($mytitlebar,"CATIA.CATNls");
    unless ( open (CATNLSORG, "< $catnlsorg") ) {
      &print_error("Cannot read file $catnlsorg");
      exit 1;
    }
    unless ( open (CATNLS, "> $catnls") ) {
      &print_error("Cannot write file $catnls");
      exit 1;
    }
	
	while (<CATNLSORG>) {
	  chomp;
	  $_="ApplicationFrame.Title = \"$tb{'my'}\";" if (/^ApplicationFrame.Title.*/);
	  print CATNLS "$_\n";
    }
    close CATNLSORG;	
    close CATNLS;	
	
    $ENV{CATMsgCatalogPath}=$mytitlebar . $seperator . $ENV{CATMsgCatalogPath};
  }
}


sub spc_start {
  my ($spc, $spctty, $spclog, $spcdir, $spctemplate, $spcstart, $pwd);
  $spc=$_[0];
  $spctty=0;
  $spctty=$_[1] if defined $_[1];
  if ( ! -f $spc && -f catfile($spc, 'xdriver') ) {
    $spc=catfile($spc, 'xdriver');
  }
  if ( $ostype eq 'unix' && `ps -e | grep  "[x]driver" | wc -l` != 0 ) {
    &print_h2("Spacemouse-driver \"xdriver\" already active!");
    return;
  }
  if ( $ostype eq 'win' && &getWinProcessId(basename($spc))>0 ) {
    &print_h2("Spacemouse-driver " . basename($spc) . " already active!");
    return;
  }
  # maybe we have only the directory name of xdriver
  if ( ! -f $spc && -f catfile($spc, 'xdriver') ) {
    $spc=catfile($spc, 'xdriver');
  }
  if ( ! -f $spc || ! -r $spc ) {
    &print_warn("Cannot find spacemouse-driver $spc, will skip this option!");
  }
  elsif ( ! &isExecutable($spc) ) {
    &print_warn("Spacemouse-driver $spc is not executable, will skip this option!");
  }
  elsif ( $ostype eq 'unix' && (basename $spc) ne 'xdriver' ) {
    &print_warn("Only spacemouse-driver \"xdriver\" is supported , will skip this option!");
  }
  else {
    $pwd=getcwd();
    if ( $ostype eq 'unix' ) {
      # compatibility to V4-SCT-installation
      $spcdir=catfile($home, 'user', $plattform);
      $spctemplate=catfile(dirname($spc), 'xdriver.ini');
      $spclog=catfile($home, 'xdriver.log');
      if ( ! -d $spcdir ) {
        unless ( mkpath($spcdir, 0, 0777) ) {
          &print_error("Cannot ceate directory $spcdir for spacemouse support!");
          exit 1;
	      }
      }
      chdir $spcdir;
      if ( -l "xdriver" && ! -f "xdriver" ) {
        unlink "xdriver";
      }
      if ( ! -l "xdriver" ) {
        symlink $spc, "xdriver";
      }
      
      # check if spctty is the full device name (preferred) or
      #                    only a number (compatibility for older versions)
      if ( $spctty=~/^\d+$/ ) {
        # not a device name, only a number
        for ($plattform) {
          /AIX/    && do { $spctty="/dev/tty$spctty" };
          /HP-UX/  && do { $spctty="/dev/tty0p$spctty" };
          /IRIX/i  && do { $spctty="/dev/ttyd$spctty" };
          /SunOS/i && do { $spctty=~tr /0-7/a-h/ ; $spctty="/dev/term/$spctty" };
        }
      }
      if ( ! -e $spctty ) {
        &print_warn("Device $spctty not found!");
      }
      for ($plattform) {
        /AIX/ && do {
#          $spcstart="./xdriver -serialport /dev/tty$spctty 1>$spclog 2>\&1 \&" }; # this kills the driver after CATIA exits
          $spcstart="ksh -i \"./xdriver -serialport $spctty 1>$spclog 2>\&1\"\&" };
        /HP-UX/ && do {
          $spcstart="ksh -i \"./xdriver -serialport $spctty 1>$spclog 2>\&1\"\&" };
        /IRIX/i && do {
          $spcstart="ksh -i \"./xdriver -serialport $spctty 1>$spclog 2>\&1\"\&" };
        /SunOS/i && do {
	  $spcstart="ksh -i \"./xdriver -serialport $spctty 1>$spclog 2>\&1\"\&" };
      }
      if ( ! -f 'xdriver.ini' ) {
        if ( -r $spctemplate ) {
	  copy ( $spctemplate, "xdriver.ini" ) ;
          chmod 0666, 'xdriver.ini';
	}
	else {
	  $spcstart ="./xdriver -new 1>$spclog 2>\&1 \&";
 	}
      }
      system "$spcstart";
      sleep 10;
    }
    else { # WIN
      system "start /B $spc";
      sleep 5;
    }
    chdir $pwd;
  }
}


sub print_envdump {
  my ($dump, $key);
  $dump="$_[0]";
  open ENVDUMP, "> $dump";
  foreach $key (sort keys(%ENV)) {
    print ENVDUMP "$key=$ENV{$key}\n";
  }
  close ENVDUMP;
}


sub getCatiaPath {
  # returns the original path for DS binaries, e.g. "c:\catia\B17BMW\intel_a"
  my ($ret, $dummy); my $found=0;
  foreach $dummy (split /$seperator+/, $ENV{CATInstallPath}) {
    foreach (glob(catfile($dummy, "*.lp")) ) {
	  if ( -f catfile($dummy, "code", "bin", "CATSTART.exe" ) || \
	       -f catfile($dummy, "code", "command", "catstart" ) ) {
        $ret=$dummy;
	    $found=1;
        last;
	  }
    }
	last if $found;
  }
  &print_error("Cannot validate CATIAPATH (*.lp file), please check!") if ! $found;
  return $ret;
}


sub getNXPath {
  return $ENV{UGII_BASE_DIR};
}


sub getTimeStamp {
  # returns a string for DATE
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$ydat,$isdst)=localtime();
  $year+=1900;
  $mon+=1;
  $mon="0$mon"		if (length($mon) == 1);
  $mday="0$mday"	if(length($mday) == 1);
  $hour="0$hour"	if(length($hour) == 1);
  $min="0$min"		if(length($min) == 1);
  $sec="0$sec"		if(length($sec) == 1);

  return ($year.$mon.$mday.$hour.$min.$sec);
}


sub getDateAndTime {
  # returns a string for DATE and TIME
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$ydat,$isdst)=localtime();
  $year+=1900;
  $mon+=1;
  $mon="0$mon"		if (length($mon) == 1);
  $mday="0$mday"	if(length($mday) == 1);
  $hour="0$hour"	if(length($hour) == 1);
  $min="0$min"		if(length($min) == 1);
  $sec="0$sec"		if(length($sec) == 1);

  return ($mday.".".$mon.".".$year, $hour.":".$min.":".$sec);
}


sub getWinProcessId {
  # to call with &getWinProcessId(ProcName)
  # returns the process id
  # returns -1 if no process found
  # returns -2 if WMI not available
  # require Win32::OLE qw( in );
  my $ProcName=$_[0];
  my $Class = "Win32_Process";
  (my $Machine = shift @ARGV || "." ) =~ s/^[\\\/]+//;
  if( my $WMIServices = Win32::OLE-> GetObject(
  "winmgmts:{impersonationLevel=impersonate,(security)}//$Machine" ) ) {
    foreach my $Processes ( in( $WMIServices-> InstancesOf( $Class ) ) ) {
      if ( "$Processes->{Name}"=~/$ProcName/i ) {
        return ($Processes-> {ProcessId});
      }
    }
    return -1;
  }
  else
  {
    # WMI does not work
    return -2;
  }
}


sub get_kerneltype {
  # returns 32|64
  my $type;
  for ($plattform) {
    /AIX/    && do { $type=`/usr/sbin/getsystype -K`; };
    /HP-UX/  && do { 
                     $type=`getconf KERNEL_BITS 2>/dev/null`;
                     $type=32 if $? == 512;
                   };
    /IRIX/i  && do {  };
    /SunOS/i && do {  };
  }
  return $type;
}


sub isExecutable {
  # -x don't work on WIN
  if ( $ostype eq 'unix' ) {
    if ( -x $_[0] ) { 1 } else { 0 }
  }
  else { 1 }
}


sub print_h1 {
  my $l=80;
  print "\n";
  if ( ! defined $_[0] ) {
    print "=" x $l, "\n";
  }
  else {
    print "======== $_[0] ", "=" x ($l-length("======== $_[0] ")), "\n";
  }
}
sub print_info {
  my $c=1;
  $c=$_[1] if defined $_[1];
  print "  " x $c, "$_[0]\n";
}
sub print_warn {
  my $c=1;
  $c=$_[1] if defined $_[1];
  print "  " x $c, "WARNING: $_[0]\n";
}
sub print_error {
  my $c=1;
  $c=$_[1] if defined $_[1];
  print "  " x $c, "ERROR: $_[0]\n";
}
sub print_h2 {
  my $c=2;
  $c=$_[1] if defined $_[1];
  print "  " x $c, "$_[0]\n";
}

sub check_required_perl {
  if ( $] < $RequiredPerlVersion ) {
    print "\n";
    print "  !!! Please install a newer perl version !!!\n";
	print "        Your perl version:    $]\n";
    print "         Required version: >= $RequiredPerlVersion\n";
    sleep 30;
    exit 1;
  }
}

sub check_load_modules {
  # to call like &check_load_modules(\@ARRAY);
  my @modules=@{$_[0]};
  foreach ( @modules ) {
    eval "use $_";
    if($@) {
      print "Modul $_ not installed. Please check your Perl installation!\n";
	  exit 1
    }    
  }
}

sub print_schwindt {
  printf "\n%-27s (c)2020 Schwindt CAD/CAM-Technologie GmbH\n",$MyName . " (v" . $MyVersion . "),";
  printf "%-45sCallenberger Str. 8\n"," ";
  printf "%-45sD-96450 Coburg\n"," ";
  printf "%-45sGermany\n"," ";
  printf "%-45sPhone  +49 9561 5560 0\n"," ";
  printf "%-45smailto info\@schwindt.eu\n\n\n"," ";
  printf "Workstation \"$hostname\", OS \"$plattform\", Perl %vd\n\n", $^V;
  printf "  command-line options: $cmdopt\n";
}

sub printusage {
  # like a manpage
  &print_schwindt;
  print "\n";
  print " NAME\n";
  print "   $MyName - starts CATIA and NX on UNIX and WINDOWS plattforms\n";
  print "\n";
  print " SYNTAX\n";
  printf"   %-15s[-env environment-name]            \\\n",$MyName;
  print "                  [-direnv environment-directory]    \\\n";
  print "                  [-object object]                   \\\n";
  print "                  [-remote hostname]                 \\\n";
  print "                  [-run program_name]                \\\n";
  print "                  [-user user_name]                  \\\n";
  print "                  [-s]                               \\\n";
  print "                  [-h]                               \\\n";
  print "---- additionally options, not supported by original script -----\n";
  print "                  [-project|p \"project1 projectn\"]   \\\n";
  print "                  [-catref \"catref1:catrefn\"]        \\\n";
  print "                  [-reset]                           \\\n";
  print "                  [-lic licname|licfile]             \\\n";
  print "                  [-spc spacemouse_driver]           \\\n";
  print "                  [-spctty spacemouse_tty]           \\\n";
  print "                  [-nocaa]                           \\\n";
  print "                  [-noscripts]                       \\\n";
  print "                  [-nodlnames]                       \\\n";
  print "                  [-nohotfix]                        \\\n";
  print "                  [-novar]\n";
  print "\n";
  print " DESCRIPTION\n";
  print "   This script starts CATIA and NX. It runs on UNIX and WINDOWS plattforms.\n";
  print "   The functionality is similar to the original CATIA catstart but not\n";
  print "   identically.\n";
  print "   Some options are treated like in the original script.\n";
  print "   For NX you may generate a CATIA like environment text file with at least\n";
  print "   the variables UGII_BASE_DIR, UGII_ROOT_DIR, UGII_LANG.\n";
  print "\n";
  print " OPTIONS\n";
  print "   -env       followed by environment-name\n";
  print "              if not specified the script uses the shell variable\n";
  print "              V5ENV if set\n";
  print "\n";
  print "   -direnv    followed by environment-directory\n";
  print "              if not specified the script uses the shell variable\n";
  print "              V5ENVDIR if set\n";
  print "\n";
  print "   -object    followed by object to load when starting CATIA V5 or followed\n";
  print "              by arguments between \"\" (-object \"arg1 arg2    argn\")\n";
  print "              suitable only with CATIA V5\n";  
  print "\n";
  print "   -remote    followed by the name of the remote machine on which\n";
  print "              the program is run\n";
  print "              suitable only with CATIA V5\n";  
  print "\n";
  print "   -run       followed by the program to start\n";
  print "              if not specified CNEXT.exe or UGRAF.exe is started\n";
  print "\n";
  print "   -user      followed by the name of the user (only with -remote option)\n";
  print "              suitable only with CATIA V5\n";  
  print "\n";
  print "   -s         silent mode (not really supported)\n";
  print "\n";
  print "   -h         print this help\n";
  print "\n";
  print "            --- additional options ---\n";
  print "\n";
  print "   -project   followed by additional environments between \"\" for evaluation\n";
  print "              suitable only with CATIA\n";  
  print "\n";
  print "   -catref    followed by specific CATReferenceSettingPath between \"\"\n";
  print "              \"?\" is supported and ask interactive which path to use\n";
  print "              suitable only with CATIA\n";  
  print "\n";
  print "   -reset     delete CATUserSettings\n";
  print "\n";
  print "   -lic       followed by V5 license name or file\n";
  print "              a name (e.g. MD2) will be imported by xml import mechanism\n";
  print "              a file ending \".CATSettings\" is copied to Licensing.CATSettings\n";
  print "              suitable only with CATIA V5\n";  
  print "\n";
  print "   -spc       followed by spacemouse driver to start\n";
  print "\n";
  print "   -spctty    followed by spacemouse device, if not specified \"0\" is used\n";
  print "\n";
  print "   -nocaa     ignore V5CAA, see below\n";
  print "              suitable only with CATIA V5\n";  
  print "\n";
  print "   -noscripts ignore V5SCRIPTS and SCHWINDTSCRIPTS, see below\n";
  print "\n";
  print "   -nodlnames ignore V5DLANMES, see below\n";
  print "              suitable only with CATIA V5\n";  
  print "\n";
#  we do not really support this
#  print "   -nohotfix  ignore V5HOTFIX, see below\n";
#  print "              suitable only with CATIA V5\n";  
#  print "\n";
#  print "   -novar     all the 4 previous options together\n";
  print "   -novar     all the 3 previous options together\n";
  print "\n";
  print "   After setting the environment, the script looks for the following variables.\n";
  print "   We recommend, to set these variables directly in the environment-file.\n";
  print "   The syntax is like the PATH variable.\n";
  print "\n";
  print "   V5CAA\n";
  print "     A list of paths with caa programs.\n";
  print "     Before starting CATIA, all scripts beginning with \"start_\*.pl\" found\n";
  print "     in subfolders of these paths will be executed.\n";
  print "     After CATIA ends, same behavior with scripts beginning with \"stop_\*.pl\".\n";
  print "     This feature may help you to include third party CAA programs.\n";
  print "\n";
  print "   V5SCRIPTS|SCHWINDTSCRIPTS\n";
  print "     A list of paths. The functionality is similar to V5CAA, but scripts found\n";
  print "     in these folders (not subfolders) will be executed\n";
  print "\n";
  print "   V5DLNAMES\n";
  print "     A list of files. These files are used for dlname import.\n";
  print "     Additional user defined files are supported. The script looks for\n";
  print "     \$HOME/.catia/mydlnames.txt and \$HOME/.catia/mydlnames_[ENV]\n";
  print "     Also V4 CATIA.dcls files are supported! (useful for UNIX)\n";
  print "     suitable only with CATIA V5\n";  
  print "\n";
  #print "   V5HOTFIX\n";
  #print "     A directory with a hotfix (installed in a special way in a new directory).\n";
  #print "     The hotfix is inserted automatically in the environment.\n";
  #print "\n";
  print "\n";
  print "   If the variables CATLOWPRIORITY=1 / NXLOWPRIORITY=1 are detected the\n";
  print "   CATIA / NX process is started with lowest priority. May be you'll need this\n";
  print "   for processes which runs on all cores and the usability for other\n";
  print "   applications suffers.\n";
  print "\n";
}
